# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CVS_CHECK_METADATA = Metadata(
    id="19f10a0aa62b259f8e415446e94d3a10512ce5d1.boutiques",
    name="mri_cvs_check",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCvsCheckOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_cvs_check(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_cvs_check(
    mov_subjid: str,
    template_subjid: str | None = None,
    hemi: typing.Literal["lh", "rh"] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MriCvsCheckOutputs:
    """
    Checks whether the files required for mri_cvs_register all exist.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mov_subjid: Subject id for the subject to be moved/registered (Should\
            be present in SUBJECTS_DIR).
        template_subjid: Subject id for the template subject to be kept fixed.\
            If missing, CVS template is assumed as a target.
        hemi: The hemisphere that is going to be processed. It can be 'lh' or\
            'rh'.
        help_: Print help and exit.
        version: Print version and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCvsCheckOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CVS_CHECK_METADATA)
    cargs = []
    cargs.append("mri_cvs_check")
    cargs.extend([
        "--mov",
        mov_subjid
    ])
    if template_subjid is not None:
        cargs.extend([
            "--template",
            template_subjid
        ])
    if hemi is not None:
        cargs.extend([
            "--hemi",
            hemi
        ])
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    ret = MriCvsCheckOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CVS_CHECK_METADATA",
    "MriCvsCheckOutputs",
    "mri_cvs_check",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REINFLATE_SUBJECT_LH_METADATA = Metadata(
    id="3a29a9cc35621246bc04494a51e4c6898d34911c.boutiques",
    name="reinflate_subject-lh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class ReinflateSubjectLhOutputs(typing.NamedTuple):
    """
    Output object returned when calling `reinflate_subject_lh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    inflated_surface: OutputPathType
    """Reinflated left hemisphere cortical surface"""


def reinflate_subject_lh(
    subject_id: str,
    runner: Runner | None = None,
) -> ReinflateSubjectLhOutputs:
    """
    Utility to reinflate cortical surfaces for left hemisphere in FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_id: Subject identifier.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ReinflateSubjectLhOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REINFLATE_SUBJECT_LH_METADATA)
    cargs = []
    cargs.extend([
        "-lh",
        "reinflate_subject" + subject_id
    ])
    ret = ReinflateSubjectLhOutputs(
        root=execution.output_file("."),
        inflated_surface=execution.output_file(subject_id + "/surf/lh.inflated"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REINFLATE_SUBJECT_LH_METADATA",
    "ReinflateSubjectLhOutputs",
    "reinflate_subject_lh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMCUTUP_METADATA = Metadata(
    id="f7a6e708513c2d65a4c90feab343a1b3f974642a.boutiques",
    name="imcutup",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ImcutupOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imcutup(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_files: OutputPathType | None
    """Output smaller images with the specified prefix numbering format."""


def imcutup(
    nx: int,
    ny: int,
    input_file: InputPathType,
    prefix: str | None = None,
    xynum: bool = False,
    yxnum: bool = False,
    xynum_format: bool = False,
    yxnum_format: bool = False,
    runner: Runner | None = None,
) -> ImcutupOutputs:
    """
    Breaks up larger images into smaller image files of user-defined size.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        nx: Number of pixels along the x-dimension for the smaller images.
        ny: Number of pixels along the y-dimension for the smaller images.
        input_file: Input image filename. Must be a single 2D image.
        prefix: Prefix the output files with the provided string.
        xynum: Number the output images in x-first, then y (default behavior).
        yxnum: Number the output images in y-first, then x.
        xynum_format: 2D numbering in x.y format.
        yxnum_format: 2D numbering in y.x format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImcutupOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMCUTUP_METADATA)
    cargs = []
    cargs.append("imcutup")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if xynum:
        cargs.append("-xynum")
    if yxnum:
        cargs.append("-yxnum")
    if xynum_format:
        cargs.append("-x.ynum")
    if yxnum_format:
        cargs.append("-y.xnum")
    cargs.append(str(nx))
    cargs.append(str(ny))
    cargs.append(execution.input_file(input_file))
    ret = ImcutupOutputs(
        root=execution.output_file("."),
        output_files=execution.output_file(prefix + "*") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMCUTUP_METADATA",
    "ImcutupOutputs",
    "imcutup",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SPHARM_DECO_METADATA = Metadata(
    id="6c5408eda75c3fcbee3c137d9a99cb7b538fb149.boutiques",
    name="SpharmDeco",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SpharmDecoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `spharm_deco(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    harmonics_file: OutputPathType
    """File for harmonics of each order l."""
    beta_coefficients: OutputPathType
    """Beta coefficients for each data column."""
    reconstructed_data: OutputPathType
    """Reconstructed data or surface files named based on PREFIX."""


def spharm_deco(
    i_type_s: InputPathType,
    unit_sph_label: str,
    order_l: float,
    i_type_sd: list[InputPathType] | None = None,
    data_d: InputPathType | None = None,
    bases_prefix: str | None = None,
    prefix: str | None = None,
    o_type_sdr: list[InputPathType] | None = None,
    debug: float | None = None,
    sigma: float | None = None,
    runner: Runner | None = None,
) -> SpharmDecoOutputs:
    """
    Spherical Harmonics Decomposition of a surface's coordinates or data.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        i_type_s: Unit sphere, isotopic to the surface domain over which the\
            data to be decomposed is defined.
        unit_sph_label: Label of the unit sphere.
        order_l: Decomposition order.
        i_type_sd: A surface whose node coordinates provide data vectors (X, Y,\
            Z) to be decomposed or a dataset whose columns are to be individually\
            decomposed. You can specify multiple surfaces to be processed.
        data_d: Data vectors to be decomposed.
        bases_prefix: Save the basis functions under the prefix BASES_PREFIX.
        prefix: Write out the reconstructed data into dataset PREFIX and write\
            the beta coefficients for each processed data column.
        o_type_sdr: Write out a new surface with reconstructed coordinates.
        debug: Debug levels (1-3).
        sigma: Smoothing parameter (0 .. 0.001) which weighs down the\
            contribution of higher order harmonics.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpharmDecoOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SPHARM_DECO_METADATA)
    cargs = []
    cargs.append("SpharmDeco")
    cargs.append(execution.input_file(i_type_s))
    cargs.append(unit_sph_label)
    cargs.append(str(order_l))
    if i_type_sd is not None:
        cargs.extend([execution.input_file(f) for f in i_type_sd])
    if data_d is not None:
        cargs.append(execution.input_file(data_d))
    if bases_prefix is not None:
        cargs.append(bases_prefix)
    if prefix is not None:
        cargs.append(prefix)
    if o_type_sdr is not None:
        cargs.extend([execution.input_file(f) for f in o_type_sdr])
    if debug is not None:
        cargs.extend([
            "-debug",
            str(debug)
        ])
    if sigma is not None:
        cargs.extend([
            "-sigma",
            str(sigma)
        ])
    ret = SpharmDecoOutputs(
        root=execution.output_file("."),
        harmonics_file=execution.output_file("BASES_PREFIX.sph*.1D"),
        beta_coefficients=execution.output_file("PREFIX.beta.col*.1D.dset"),
        reconstructed_data=execution.output_file("<PREFIX>_reconstructed"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SPHARM_DECO_METADATA",
    "SpharmDecoOutputs",
    "spharm_deco",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__NOISY_SKULL_STRIP_METADATA = Metadata(
    id="a95b392481360cc4893886c77cddfcea221759d0.boutiques",
    name="@NoisySkullStrip",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VNoisySkullStripOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__noisy_skull_strip(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    anat_ns: OutputPathType
    """Skull stripped version of ANAT"""
    anat_air: OutputPathType
    """Special mask output - air"""
    anat_skl: OutputPathType
    """Special mask output - skull"""
    anat_lsp: OutputPathType
    """Volume used to threshold 'air' out of the volume to be stripped"""


def v__noisy_skull_strip(
    input_file: InputPathType,
    keep_tmp: bool = False,
    v_3dskullstrip_opts: str | None = None,
    runner: Runner | None = None,
) -> VNoisySkullStripOutputs:
    """
    Strips the skull of anatomical datasets with low SNR.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_file: The anatomical dataset.
        keep_tmp: Do not erase temporary files at the end.
        v_3dskullstrip_opts: Anything following this option is passed to\
            3dSkullStrip.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VNoisySkullStripOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__NOISY_SKULL_STRIP_METADATA)
    cargs = []
    cargs.append("@NoisySkullStrip")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    if keep_tmp:
        cargs.append("-keep_tmp")
    if v_3dskullstrip_opts is not None:
        cargs.extend([
            "-3dSkullStrip_opts",
            v_3dskullstrip_opts
        ])
    ret = VNoisySkullStripOutputs(
        root=execution.output_file("."),
        anat_ns=execution.output_file(pathlib.Path(input_file).name + ".ns"),
        anat_air=execution.output_file(pathlib.Path(input_file).name + ".air"),
        anat_skl=execution.output_file(pathlib.Path(input_file).name + ".skl"),
        anat_lsp=execution.output_file(pathlib.Path(input_file).name + ".lsp"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VNoisySkullStripOutputs",
    "V__NOISY_SKULL_STRIP_METADATA",
    "v__noisy_skull_strip",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_EASYWARP_METADATA = Metadata(
    id="61155dbff21df2a7da1b05052571570b740944be.boutiques",
    name="mri_easywarp",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriEasywarpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_easywarp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_deformed_image: OutputPathType
    """Output deformed image"""


def mri_easywarp(
    input_image: InputPathType,
    output_image: str,
    deformation_field: InputPathType | None = None,
    nearest_neighbor: bool = False,
    num_threads: float | None = None,
    runner: Runner | None = None,
) -> MriEasywarpOutputs:
    """
    EasyReg: deep learning registration simple and easy.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_image: Input image.
        output_image: Output (deformed) image.
        deformation_field: Deformation field.
        nearest_neighbor: Use nearest neighbor (rather than linear)\
            interpolation.
        num_threads: Number of cores to be used. Default is 1. You can use -1\
            to use all available cores.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriEasywarpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_EASYWARP_METADATA)
    cargs = []
    cargs.append("mri_easywarp")
    cargs.extend([
        "--i",
        execution.input_file(input_image)
    ])
    cargs.extend([
        "--o",
        output_image
    ])
    if deformation_field is not None:
        cargs.extend([
            "--field",
            execution.input_file(deformation_field)
        ])
    if nearest_neighbor:
        cargs.append("--nearest")
    if num_threads is not None:
        cargs.extend([
            "--threads",
            str(num_threads)
        ])
    ret = MriEasywarpOutputs(
        root=execution.output_file("."),
        output_deformed_image=execution.output_file(output_image + ".nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_EASYWARP_METADATA",
    "MriEasywarpOutputs",
    "mri_easywarp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_COMPUTE_OVERLAP_METADATA = Metadata(
    id="a5b5b5d3058860c230b1e6a107208c5201cd453b.boutiques",
    name="mris_compute_overlap",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisComputeOverlapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_compute_overlap(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mris_compute_overlap(
    subject: str,
    hemi: str,
    surface: str,
    annotation: str,
    labels: list[str],
    percentage: bool = False,
    log_file: str | None = None,
    brain_volume: InputPathType | None = None,
    runner: Runner | None = None,
) -> MrisComputeOverlapOutputs:
    """
    Tool to compute the overlap between two or more labels on a cortical surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: Subject name.
        hemi: Hemisphere (e.g. lh or rh).
        surface: Surface name.
        annotation: Annotation name.
        labels: Labels to compute overlap for.
        percentage: Compute brain area as a percentage of all brain labels.
        log_file: Log results to file, where %d will include label number.
        brain_volume: Load brain volume and use it to normalize areas.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisComputeOverlapOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_COMPUTE_OVERLAP_METADATA)
    cargs = []
    cargs.append("mris_compute_overlap")
    cargs.append(subject)
    cargs.append(hemi)
    cargs.append(surface)
    cargs.append(annotation)
    cargs.extend(labels)
    if percentage:
        cargs.append("-p")
    if log_file is not None:
        cargs.extend([
            "-l",
            log_file
        ])
    if brain_volume is not None:
        cargs.extend([
            "-b",
            execution.input_file(brain_volume)
        ])
    ret = MrisComputeOverlapOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_COMPUTE_OVERLAP_METADATA",
    "MrisComputeOverlapOutputs",
    "mris_compute_overlap",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_SMOOTH_METADATA = Metadata(
    id="75bc8e87c534738bfed1dc6d67307d0c1e29317a.boutiques",
    name="mris_smooth",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisSmoothOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_smooth(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_surface_file: OutputPathType
    """Output smoothed surface file."""
    curvature_file: OutputPathType
    """Output curvature file (if written)."""
    area_file: OutputPathType
    """Output area file (if written)."""


def mris_smooth(
    input_surface: InputPathType,
    output_surface: str,
    average_iters: float | None = 10,
    smoothing_iters: float | None = 10,
    no_write: bool = False,
    curvature_name: str | None = "curv",
    area_name: str | None = "area",
    gaussian_params: list[float] | None = None,
    normalize_area: bool = False,
    momentum: float | None = None,
    snapshot_interval: float | None = None,
    runner: Runner | None = None,
) -> MrisSmoothOutputs:
    """
    This program smooths the tessellation of a cortical surface and writes out the
    mean curvature and area files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: Input surface file for smoothing.
        output_surface: Output surface file after smoothing.
        average_iters: Specify number of curvature averaging iterations\
            (default is 10).
        smoothing_iters: Specify number of smoothing iterations (default is\
            10).
        no_write: Disable writing of curvature and area estimates.
        curvature_name: Write curvature to a specified file name (default\
            'curv').
        area_name: Write area to a specified file name (default 'area').
        gaussian_params: Use Gaussian curvature smoothing with specified norm\
            and steps.
        normalize_area: Normalize area after smoothing.
        momentum: Set momentum value.
        snapshot_interval: Write snapshot every specified number of iterations.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisSmoothOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_SMOOTH_METADATA)
    cargs = []
    cargs.append("mris_smooth")
    cargs.append(execution.input_file(input_surface))
    cargs.append(output_surface)
    if average_iters is not None:
        cargs.extend([
            "-a",
            str(average_iters)
        ])
    if smoothing_iters is not None:
        cargs.extend([
            "-n",
            str(smoothing_iters)
        ])
    if no_write:
        cargs.append("-nw")
    if curvature_name is not None:
        cargs.extend([
            "-c",
            curvature_name
        ])
    if area_name is not None:
        cargs.extend([
            "-b",
            area_name
        ])
    if gaussian_params is not None:
        cargs.extend([
            "-g",
            *map(str, gaussian_params)
        ])
    if normalize_area:
        cargs.append("-area")
    if momentum is not None:
        cargs.extend([
            "-m",
            str(momentum)
        ])
    if snapshot_interval is not None:
        cargs.extend([
            "-w",
            str(snapshot_interval)
        ])
    ret = MrisSmoothOutputs(
        root=execution.output_file("."),
        output_surface_file=execution.output_file(output_surface),
        curvature_file=execution.output_file("${OUTPUT_SURFACE}_curvature"),
        area_file=execution.output_file("${OUTPUT_SURFACE}_area"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_SMOOTH_METADATA",
    "MrisSmoothOutputs",
    "mris_smooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_EXPAND_METADATA = Metadata(
    id="b9cee6836960fc68a28400b950ad233fa015f146.boutiques",
    name="mris_expand",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisExpandOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_expand(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_surface_file: OutputPathType
    """Expanded output surface file"""


def mris_expand(
    input_surface: InputPathType,
    expansion_distance: float,
    output_surface: str,
    thickness: bool = False,
    label: str | None = None,
    tmap: str | None = None,
    tmap_random: str | None = None,
    runner: Runner | None = None,
) -> MrisExpandOutputs:
    """
    Expand a given surface by a specified distance.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: Input surface file (e.g. lh.white).
        expansion_distance: Expansion distance in mm.
        output_surface: Output surface file.
        thickness: Use thickness for expansion.
        label: Use label file for expansion.
        tmap: Use a prespecified map of percent thickness to compute the target\
            locations for expansion.
        tmap_random: Create a random target distance map with Gaussian sampling\
            for the target locations.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisExpandOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_EXPAND_METADATA)
    cargs = []
    cargs.append("mris_expand")
    cargs.append(execution.input_file(input_surface))
    cargs.append(str(expansion_distance))
    cargs.append(output_surface)
    if thickness:
        cargs.append("-thickness")
    if label is not None:
        cargs.extend([
            "-label",
            label
        ])
    if tmap is not None:
        cargs.extend([
            "-tmap",
            tmap
        ])
    if tmap_random is not None:
        cargs.extend([
            "-tmap random",
            tmap_random
        ])
    ret = MrisExpandOutputs(
        root=execution.output_file("."),
        output_surface_file=execution.output_file(output_surface),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_EXPAND_METADATA",
    "MrisExpandOutputs",
    "mris_expand",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DEFECT_SEG_METADATA = Metadata(
    id="0e4a4cf0254a2348ea18fc134e746047a23ee4d4.boutiques",
    name="defect-seg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class DefectSegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `defect_seg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_defects: OutputPathType
    """Surface defects volume."""
    defects_summary: OutputPathType
    """Summary of defects indicating numerical ID and number of vertices."""
    defect_labels_fix: OutputPathType
    """Fixed defect labels on the surface."""
    defect_labels_fix_bin: OutputPathType
    """Binary labels for fixed defects."""
    defects_nofix_annot: OutputPathType
    """Annotation file for defects without fixes."""
    defects_fix_annot: OutputPathType
    """Annotation file for defects with fixes."""
    defects_stats: OutputPathType
    """Statistics summary of defects indicating area and average thickness."""


def defect_seg(
    subject: str,
    lh_only: bool = False,
    rh_only: bool = False,
    runner: Runner | None = None,
) -> DefectSegOutputs:
    """
    This program creates some files that allows the user to visualize and evaluate
    the surface defects that were automatically detected and fixed by the topology
    correction program.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: Subject identifier.
        lh_only: Only process the left hemisphere.
        rh_only: Only process the right hemisphere.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DefectSegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DEFECT_SEG_METADATA)
    cargs = []
    cargs.append("defect-seg")
    cargs.extend([
        "--s",
        subject
    ])
    if lh_only:
        cargs.append("--lh-only")
    if rh_only:
        cargs.append("--rh-only")
    ret = DefectSegOutputs(
        root=execution.output_file("."),
        surface_defects=execution.output_file("mri/surface.defects.mgz"),
        defects_summary=execution.output_file("scripts/?h.defects.summary"),
        defect_labels_fix=execution.output_file("surf/?h.defect_labels.fix.mgz"),
        defect_labels_fix_bin=execution.output_file("surf/?h.defect_labels.fix.bin.mgz"),
        defects_nofix_annot=execution.output_file("label/?h.defects.nofix.annot"),
        defects_fix_annot=execution.output_file("label/?h.defects.fix.annot"),
        defects_stats=execution.output_file("stats/?h.defects.stats"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DEFECT_SEG_METADATA",
    "DefectSegOutputs",
    "defect_seg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_CROSS_CORRELATION_METADATA = Metadata(
    id="d6e5609483ce1b783c981ae6ed17a12d344a7123.boutiques",
    name="cifti-cross-correlation",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class CiftiCrossCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_cross_correlation(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_cross_correlation(
    cifti_a: InputPathType,
    cifti_b: InputPathType,
    cifti_out: str,
    opt_weights_weight_file: str | None = None,
    opt_fisher_z: bool = False,
    opt_mem_limit_limit_gb: float | None = None,
    runner: Runner | None = None,
) -> CiftiCrossCorrelationOutputs:
    """
    Correlate a cifti file with another cifti file.
    
    Correlates every row in <cifti-a> with every row in <cifti-b>. The mapping
    along columns in <cifti-b> becomes the mapping along rows in the output.
    
    When using the -fisher-z option, the output is NOT a Z-score, it is
    artanh(r), to do further math on this output, consider using -cifti-math.
    
    Restricting the memory usage will make it calculate the output in chunks, by
    reading through <cifti-b> multiple times.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_a: first input cifti file.
        cifti_b: second input cifti file.
        cifti_out: output cifti file.
        opt_weights_weight_file: specify column weights: text file containing\
            one weight per column.
        opt_fisher_z: apply fisher small z transform (ie, artanh) to\
            correlation.
        opt_mem_limit_limit_gb: restrict memory usage: memory limit in\
            gigabytes.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCrossCorrelationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CROSS_CORRELATION_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-cross-correlation")
    cargs.append(execution.input_file(cifti_a))
    cargs.append(execution.input_file(cifti_b))
    cargs.append(cifti_out)
    if opt_weights_weight_file is not None:
        cargs.extend([
            "-weights",
            opt_weights_weight_file
        ])
    if opt_fisher_z:
        cargs.append("-fisher-z")
    if opt_mem_limit_limit_gb is not None:
        cargs.extend([
            "-mem-limit",
            str(opt_mem_limit_limit_gb)
        ])
    ret = CiftiCrossCorrelationOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_CROSS_CORRELATION_METADATA",
    "CiftiCrossCorrelationOutputs",
    "cifti_cross_correlation",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SCCAN_METADATA = Metadata(
    id="26f604003de4aee5a1d0df471af2893515e5b091.boutiques",
    name="sccan",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class SccanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `sccan(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def sccan(
    output: str | None = None,
    n_permutations: int | None = None,
    smoother: int | None = None,
    row_sparseness: int | None = None,
    iterations: int | None = None,
    n_eigenvectors: int | None = None,
    robustify: int | None = None,
    covering: int | None = None,
    uselong: int | None = None,
    l1: float | None = None,
    pclusterthresh: float | None = None,
    qclusterthresh: float | None = None,
    ridge_cca: float | None = None,
    initialization: str | None = None,
    initialization2: str | None = None,
    mask: InputPathType | None = None,
    mask2: InputPathType | None = None,
    partial_scca_option: str | None = None,
    prior_weight: float | None = None,
    get_small: float | None = None,
    verbose: float | None = None,
    imageset_to_matrix: str | None = None,
    timeseriesimage_to_matrix: str | None = None,
    vector_to_image: str | None = None,
    imageset_to_projections: str | None = None,
    scca: str | None = None,
    svd: str | None = None,
    runner: Runner | None = None,
) -> SccanOutputs:
    """
    A tool for sparse statistical analysis on images : scca, pscca (with options),
    mscca. Can also convert an imagelist/mask pair to a binary matrix image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        output: Output dependent on which option is called.
        n_permutations: Number of permutations to use in scca.
        smoother: Smoothing function for variates.
        row_sparseness: Row sparseness - if (+) then keep values (+) otherwise\
            allow +/- values --- always L1.
        iterations: Max iterations for scca optimization (min 20).
        n_eigenvectors: Number of eigenvectors to compute in scca/spca.
        robustify: Rank-based scca.
        covering: Try to make the decomposition cover the whole domain, if\
            possible.
        uselong: Use longitudinal formulation (> 0) or not (<= 0).
        l1: Use l1 (> 0) or l0 (< 0) penalty, also sets gradient step size.
        pclusterthresh: Cluster threshold on view P.
        qclusterthresh: Cluster threshold on view Q.
        ridge_cca: Ridge cca.
        initialization: Initialization file list for Eigenanatomy - must also\
            pass mask option.
        initialization2: Initialization file list for SCCAN-Eigenanatomy - must\
            also pass mask option.
        mask: Mask file for Eigenanatomy initialization.
        mask2: Mask file for Eigenanatomy initialization 2.
        partial_scca_option: Choices for pscca: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        prior_weight: Scalar value weight on prior between 0 (prior is weak)\
            and 1 (prior is strong). Only engaged if initialization is used.
        get_small: Find smallest eigenvectors.
        verbose: Set whether output is verbose.
        imageset_to_matrix: Takes a list of image files names (one per line)\
            and converts it to a 2D matrix/image in binary or csv format.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format.
        vector_to_image: Converts the 1st column vector in a csv file back to\
            an image.
        imageset_to_projections: Takes a list of image and projection files\
            names and writes them to a csv file.
        scca: Matrix-based scca operations for 2 and 3 views.
        svd: A sparse SVD implementation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SccanOutputs`).
    """
    if iterations is not None and not (20 <= iterations): 
        raise ValueError(f"'iterations' must be greater than 20 <= x but was {iterations}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCCAN_METADATA)
    cargs = []
    cargs.append("sccan")
    if output is not None:
        cargs.extend([
            "-o",
            output
        ])
    if n_permutations is not None:
        cargs.extend([
            "-p",
            str(n_permutations)
        ])
    if smoother is not None:
        cargs.extend([
            "-s",
            str(smoother)
        ])
    if row_sparseness is not None:
        cargs.extend([
            "-z",
            str(row_sparseness)
        ])
    if iterations is not None:
        cargs.extend([
            "-i",
            str(iterations)
        ])
    if n_eigenvectors is not None:
        cargs.extend([
            "-n",
            str(n_eigenvectors)
        ])
    if robustify is not None:
        cargs.extend([
            "-r",
            str(robustify)
        ])
    if covering is not None:
        cargs.extend([
            "-c",
            str(covering)
        ])
    if uselong is not None:
        cargs.extend([
            "-g",
            str(uselong)
        ])
    if l1 is not None:
        cargs.extend([
            "-l",
            str(l1)
        ])
    if pclusterthresh is not None:
        cargs.extend([
            "--PClusterThresh",
            str(pclusterthresh)
        ])
    if qclusterthresh is not None:
        cargs.extend([
            "--QClusterThresh",
            str(qclusterthresh)
        ])
    if ridge_cca is not None:
        cargs.extend([
            "-e",
            str(ridge_cca)
        ])
    if initialization is not None:
        cargs.extend([
            "--initialization",
            initialization
        ])
    if initialization2 is not None:
        cargs.extend([
            "--initialization2",
            initialization2
        ])
    if mask is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask)
        ])
    if mask2 is not None:
        cargs.extend([
            "--mask2",
            execution.input_file(mask2)
        ])
    if partial_scca_option is not None:
        cargs.extend([
            "--partial-scca-option",
            partial_scca_option
        ])
    if prior_weight is not None:
        cargs.extend([
            "--prior-weight",
            str(prior_weight)
        ])
    if get_small is not None:
        cargs.extend([
            "--get-small",
            str(get_small)
        ])
    if verbose is not None:
        cargs.extend([
            "-v",
            str(verbose)
        ])
    if imageset_to_matrix is not None:
        cargs.extend([
            "--imageset-to-matrix",
            imageset_to_matrix
        ])
    if timeseriesimage_to_matrix is not None:
        cargs.extend([
            "--timeseriesimage-to-matrix",
            timeseriesimage_to_matrix
        ])
    if vector_to_image is not None:
        cargs.extend([
            "--vector-to-image",
            vector_to_image
        ])
    if imageset_to_projections is not None:
        cargs.extend([
            "--imageset-to-projections",
            imageset_to_projections
        ])
    if scca is not None:
        cargs.extend([
            "--scca",
            scca
        ])
    if svd is not None:
        cargs.extend([
            "--svd",
            svd
        ])
    ret = SccanOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SCCAN_METADATA",
    "SccanOutputs",
    "sccan",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_MULTISCALE_STATS_METADATA = Metadata(
    id="70485123c5f0182ca6af11b7463e3a674edf9419.boutiques",
    name="mris_multiscale_stats",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisMultiscaleStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_multiscale_stats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mris_multiscale_stats(
    output_subject: str,
    hemi: str,
    surf: InputPathType,
    curv: InputPathType,
    class1_subjects: list[str],
    class2_subjects: list[str],
    runner: Runner | None = None,
) -> MrisMultiscaleStatsOutputs:
    """
    Compute the autocorrelation function of a curvature file using multiscale
    statistical techniques.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_subject: The output subject identifier.
        hemi: Specify which hemisphere to use.
        surf: A spherical surface file suitable for computing geodesics.
        curv: The curvature file to be processed.
        class1_subjects: List of subjects from one class.
        class2_subjects: List of subjects from another class.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisMultiscaleStatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_MULTISCALE_STATS_METADATA)
    cargs = []
    cargs.append("mris_multiscale_stats")
    cargs.extend([
        "-o",
        output_subject
    ])
    cargs.append(hemi)
    cargs.append(execution.input_file(surf))
    cargs.append(execution.input_file(curv))
    cargs.extend(class1_subjects)
    cargs.extend(class2_subjects)
    ret = MrisMultiscaleStatsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_MULTISCALE_STATS_METADATA",
    "MrisMultiscaleStatsOutputs",
    "mris_multiscale_stats",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

LABEL_SUBJECT_FLASH_METADATA = Metadata(
    id="6437bdd8443156c21638b016c6209131b9efc309.boutiques",
    name="label_subject_flash",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class LabelSubjectFlashOutputs(typing.NamedTuple):
    """
    Output object returned when calling `label_subject_flash(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    aseg_outfile: OutputPathType
    """Automatic segmentation (aseg) output file"""


def label_subject_flash(
    tissue_params: InputPathType,
    norm_volume: InputPathType,
    transform_file: InputPathType,
    classifier_array: InputPathType,
    aseg_output: str,
    runner: Runner | None = None,
) -> LabelSubjectFlashOutputs:
    """
    A tool for labeling brain structures in an MRI dataset using FLASH sequences and
    the FreeSurfer software.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        tissue_params: Path to the tissue parameter file for FLASH sequences.
        norm_volume: Path to the normalized T1 volume.
        transform_file: Talairach linear transform file.
        classifier_array: Path to the classifier array in GCA format.
        aseg_output: Output path for the automatic segmentation (aseg) file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelSubjectFlashOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_SUBJECT_FLASH_METADATA)
    cargs = []
    cargs.append("mri_ca_label")
    cargs.extend([
        "-flash",
        execution.input_file(tissue_params)
    ])
    cargs.append(execution.input_file(norm_volume))
    cargs.append(execution.input_file(transform_file))
    cargs.append(execution.input_file(classifier_array))
    cargs.append(aseg_output)
    ret = LabelSubjectFlashOutputs(
        root=execution.output_file("."),
        aseg_outfile=execution.output_file(aseg_output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "LABEL_SUBJECT_FLASH_METADATA",
    "LabelSubjectFlashOutputs",
    "label_subject_flash",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

APARC_STATS_ASEG_METADATA = Metadata(
    id="f369685392146e6aac3bec57244b528c1438fc58.boutiques",
    name="aparc_stats_aseg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class AparcStatsAsegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `aparc_stats_aseg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def aparc_stats_aseg(
    subject_name: str,
    gcs_name: str,
    subject_dir: str | None = None,
    gcs_dir: str | None = None,
    parc_name: str | None = None,
    output_dir: str | None = None,
    log_file: str | None = None,
    lh_flag: bool = False,
    rh_flag: bool = False,
    a2009s_flag: bool = False,
    no_aseg_flag: bool = False,
    no_cortparc_flag: bool = False,
    no_parcstats_flag: bool = False,
    no_aparc2aseg_flag: bool = False,
    random_seed: float | None = None,
    th3_flag: bool = False,
    no_th3_flag: bool = False,
    longitudinal: list[str] | None = None,
    expert_file: str | None = None,
    expert_use_flag: bool = False,
    expert_clean_flag: bool = False,
    expert_overwrite_flag: bool = False,
    runner: Runner | None = None,
) -> AparcStatsAsegOutputs:
    """
    This program runs mris_ca_label, mris_anatomical_stats and mri_aparc2aseg.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_name: Subject name.
        gcs_name: GCS file name.
        subject_dir: Specify a subjects directory\
            (default=/usr/local/freesurfer/subjects).
        gcs_dir: GCS file directory (default=/usr/local/freesurfer/average).
        parc_name: Specify the parcellation name (default=gcs name).
        output_dir: Specify the output directory.
        log_file: Specify the log file.
        lh_flag: Left hemisphere only.
        rh_flag: Right hemisphere only.
        a2009s_flag: Use Christophe Destrieux cortical parcellation atlas.
        no_aseg_flag: Do not use aseg.
        no_cortparc_flag: Do not do cortical parcellation.
        no_parcstats_flag: Do not do surface anatomical statistics.
        no_aparc2aseg_flag: Do not do aparc2aseg.
        random_seed: Random seed (default=1234).
        th3_flag: Use -th3 flag, turn on new volume calculation for\
            mris_anatomical_stats.
        no_th3_flag: Use -no-th3 flag for mris_anatomical_stats.
        longitudinal: Longitudinal Processing.
        expert_file: Read-in expert options file.
        expert_use_flag: Use pre-existing expert options file.
        expert_clean_flag: Delete pre-existing expert options file.
        expert_overwrite_flag: Overwrite pre-existing expert options file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AparcStatsAsegOutputs`).
    """
    if longitudinal is not None and (len(longitudinal) != 2): 
        raise ValueError(f"Length of 'longitudinal' must be 2 but was {len(longitudinal)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(APARC_STATS_ASEG_METADATA)
    cargs = []
    cargs.append("aparc_stats_aseg")
    cargs.extend([
        "-s",
        subject_name
    ])
    cargs.extend([
        "-gcs",
        gcs_name
    ])
    if subject_dir is not None:
        cargs.extend([
            "-sd",
            subject_dir
        ])
    if gcs_dir is not None:
        cargs.extend([
            "-gcsd",
            gcs_dir
        ])
    if parc_name is not None:
        cargs.extend([
            "-name",
            parc_name
        ])
    if output_dir is not None:
        cargs.extend([
            "-o",
            output_dir
        ])
    if log_file is not None:
        cargs.extend([
            "-log",
            log_file
        ])
    if lh_flag:
        cargs.append("-lh")
    if rh_flag:
        cargs.append("-rh")
    if a2009s_flag:
        cargs.append("-a2009s")
    if no_aseg_flag:
        cargs.append("-noaseg")
    if no_cortparc_flag:
        cargs.append("-nocortparc")
    if no_parcstats_flag:
        cargs.append("-noparcstats")
    if no_aparc2aseg_flag:
        cargs.append("-noaparc2aseg")
    if random_seed is not None:
        cargs.extend([
            "-seed",
            str(random_seed)
        ])
    if th3_flag:
        cargs.append("-th3")
    if no_th3_flag:
        cargs.append("-no-th3")
    if longitudinal is not None:
        cargs.extend([
            "-long",
            *longitudinal
        ])
    if expert_file is not None:
        cargs.extend([
            "-expert",
            expert_file
        ])
    if expert_use_flag:
        cargs.append("-xopts-use")
    if expert_clean_flag:
        cargs.append("-xopts-clean")
    if expert_overwrite_flag:
        cargs.append("-xopts-overwrite")
    ret = AparcStatsAsegOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "APARC_STATS_ASEG_METADATA",
    "AparcStatsAsegOutputs",
    "aparc_stats_aseg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

AP_RUN_SIMPLE_REST_METADATA = Metadata(
    id="51acc3d2305cb727e55055aa552281969a79036d.boutiques",
    name="ap_run_simple_rest",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ApRunSimpleRestOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ap_run_simple_rest(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    run_ap_script: OutputPathType | None
    """afni_proc.py command script"""
    proc_script: OutputPathType | None
    """proc script (if AP is run)"""
    proc_results_dir: OutputPathType | None
    """proc results directory (if run)"""
    text_output_files: OutputPathType
    """Text output files from AP and proc scripts"""


def ap_run_simple_rest(
    epi: list[InputPathType],
    anat: InputPathType | None = None,
    nt_rm: float | None = None,
    run_ap: bool = False,
    run_proc: bool = False,
    subjid: str | None = None,
    template: InputPathType | None = None,
    compressor: str | None = None,
    verb: float | None = None,
    echo: bool = False,
    runner: Runner | None = None,
) -> ApRunSimpleRestOutputs:
    """
    Run a quick afni_proc.py analysis for QC.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        epi: EPI datasets.
        anat: Single anatomical dataset.
        nt_rm: Number of time points to remove from starts of runs.
        run_ap: Run the afni_proc.py command.
        run_proc: Run the proc script from afni_proc.py command.
        subjid: Specify subject ID for file names.
        template: Specify template for standard space.
        compressor: Control automatic compression of *.BRIK files.
        verb: Specify verbosity level.
        echo: Same as verbosity level 3.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ApRunSimpleRestOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(AP_RUN_SIMPLE_REST_METADATA)
    cargs = []
    cargs.append("ap_run_simple_rest.tcsh")
    if anat is not None:
        cargs.extend([
            "-anat",
            execution.input_file(anat)
        ])
    cargs.extend([
        "-epi",
        *[execution.input_file(f) for f in epi]
    ])
    if nt_rm is not None:
        cargs.extend([
            "-nt_rm",
            str(nt_rm)
        ])
    if run_ap:
        cargs.append("-run_ap")
    if run_proc:
        cargs.append("-run_proc")
    if subjid is not None:
        cargs.extend([
            "-subjid",
            subjid
        ])
    if template is not None:
        cargs.extend([
            "-template",
            execution.input_file(template)
        ])
    if compressor is not None:
        cargs.extend([
            "-compressor",
            compressor
        ])
    if verb is not None:
        cargs.extend([
            "-verb",
            str(verb)
        ])
    if echo:
        cargs.append("-echo")
    ret = ApRunSimpleRestOutputs(
        root=execution.output_file("."),
        run_ap_script=execution.output_file("run_ap_" + subjid) if (subjid is not None) else None,
        proc_script=execution.output_file("proc." + subjid) if (subjid is not None) else None,
        proc_results_dir=execution.output_file(subjid + ".results") if (subjid is not None) else None,
        text_output_files=execution.output_file("out.*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "AP_RUN_SIMPLE_REST_METADATA",
    "ApRunSimpleRestOutputs",
    "ap_run_simple_rest",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MELODIC_METADATA = Metadata(
    id="676ecf3f7a3e52a01972f67049800329276421c8.boutiques",
    name="melodic",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class MelodicOutputs(typing.NamedTuple):
    """
    Output object returned when calling `melodic(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    report_file: OutputPathType | None
    """Output Melodic web report"""
    ics_output_file: OutputPathType | None
    """Output IC components file"""
    mix_output_file: OutputPathType | None
    """Output mixing matrix file"""
    temporal_mode_file: OutputPathType | None
    """Output matrix of temporal modes"""
    melodic_report_directory: OutputPathType | None
    """Output Melodic report directory"""


def melodic(
    input_file: InputPathType,
    output_directory: str | None = None,
    mask_file: InputPathType | None = None,
    dimensionality_reduction: float | None = None,
    generate_report: bool = False,
    cifti_io: bool = False,
    variance_normalization: bool = False,
    no_masking: bool = False,
    update_masking: bool = False,
    no_bet: bool = False,
    bg_threshold: float | None = None,
    dimest_technique: str | None = None,
    separate_variance_normalization: bool = False,
    disable_migp: bool = False,
    num_internal_eigenmaps: float | None = None,
    migp_shuffle: bool = False,
    migp_factor: float | None = None,
    num_ics: float | None = None,
    nonlinearity: str | None = None,
    covar_weights: InputPathType | None = None,
    eps_error: float | None = None,
    eps_rank1_error: float | None = None,
    max_iters: float | None = None,
    max_restarts: float | None = None,
    mm_threshold: float | None = None,
    no_mixture_modeling: bool = False,
    ic_components_file: InputPathType | None = None,
    mixing_matrix_file: InputPathType | None = None,
    session_modes_file: InputPathType | None = None,
    component_filter: str | None = None,
    background_image: InputPathType | None = None,
    tr_seconds: float | None = None,
    log_power_calc: bool = False,
    time_domain_design_matrix: InputPathType | None = None,
    time_domain_t_contrast_matrix: InputPathType | None = None,
    subject_domain_design_matrix: InputPathType | None = None,
    subject_domain_t_contrast_matrix: InputPathType | None = None,
    output_unmixing_matrix: bool = False,
    output_stats: bool = False,
    output_pca: bool = False,
    output_whitening: bool = False,
    output_original_ics: bool = False,
    output_mean_volume: bool = False,
    version: bool = False,
    copyright_: bool = False,
    help_: bool = False,
    debug: bool = False,
    report_maps: str | None = None,
    keep_meanvol: bool = False,
    runner: Runner | None = None,
) -> MelodicOutputs:
    """
    Multivariate Exploratory Linear Optimised Decomposition into Independent
    Components.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_file: Input file names (either single file name or\
            comma-separated list or text file).
        output_directory: Output directory name.
        mask_file: File name of mask for thresholding.
        dimensionality_reduction: Dimensionality reduction into specified\
            number of dimensions (default is automatic estimation).
        generate_report: Generate Melodic web report.
        cifti_io: Input/output as CIFTI (warning: auto-dimensionality\
            estimation for CIFTI data is currently inaccurate).
        variance_normalization: Switch off variance normalization.
        no_masking: Switch off masking.
        update_masking: Switch off mask updating.
        no_bet: Switch off BET.
        bg_threshold: Brain / non-brain threshold (only if --nobet selected).
        dimest_technique: Use specific dimensionality estimation technique:\
            lap, bic, mdl, aic, mean (default: lap).
        separate_variance_normalization: Switch on separate variance\
            normalization for each input dataset (off by default).
        disable_migp: Switch off MIGP data reduction when using -a concat (full\
            temporal concatenation will be used).
        num_internal_eigenmaps: Number of internal Eigenmaps.
        migp_shuffle: Randomize MIGP file order (default: TRUE).
        migp_factor: Internal Factor of mem-threshold relative to number of\
            Eigenmaps (default: 2).
        num_ics: Number of ICs to extract (for deflation approach).
        nonlinearity: Nonlinearity: gauss, tanh, pow3 (default), pow4.
        covar_weights: Voxel-wise weights for the covariance matrix (e.g.\
            segmentation information).
        eps_error: Minimum error change.
        eps_rank1_error: Minimum error change for rank-1 approximation in TICA.
        max_iters: Maximum number of iterations before restart.
        max_restarts: Maximum number of restarts.
        mm_threshold: Threshold for Mixture Model based inference.
        no_mixture_modeling: Switch off mixture modeling on IC maps.
        ic_components_file: Input filename of the IC components file for\
            mixture modeling.
        mixing_matrix_file: Input filename of mixing matrix for mixture\
            modeling / filtering.
        session_modes_file: Input filename of matrix of session modes for\
            report generation.
        component_filter: List of component numbers to remove.
        background_image: Specify background image for report (default: mean\
            image).
        tr_seconds: TR in seconds.
        log_power_calc: Calculate log of power for frequency spectrum.
        time_domain_design_matrix: Design matrix across time-domain.
        time_domain_t_contrast_matrix: T-contrast matrix across time-domain.
        subject_domain_design_matrix: Design matrix across subject-domain.
        subject_domain_t_contrast_matrix: T-contrast matrix across\
            subject-domain.
        output_unmixing_matrix: Output unmixing matrix.
        output_stats: Output thresholded maps and probability maps.
        output_pca: Output PCA results.
        output_whitening: Output whitening/dewhitening matrices.
        output_original_ics: Output the original ICs.
        output_mean_volume: Output mean volume.
        version: Prints version information.
        copyright_: Prints copyright information.
        help_: Prints this help message.
        debug: Switch on debug messages.
        report_maps: Control string for spatial map images (see slicer).
        keep_meanvol: Do not subtract mean volume.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MelodicOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MELODIC_METADATA)
    cargs = []
    cargs.append("melodic")
    cargs.extend([
        "-i",
        execution.input_file(input_file)
    ])
    if output_directory is not None:
        cargs.extend([
            "-o",
            output_directory
        ])
    if mask_file is not None:
        cargs.extend([
            "-m",
            execution.input_file(mask_file)
        ])
    if dimensionality_reduction is not None:
        cargs.extend([
            "-d",
            str(dimensionality_reduction)
        ])
    if generate_report:
        cargs.append("--report")
    if cifti_io:
        cargs.append("--CIFTI")
    if variance_normalization:
        cargs.append("--vn")
    if no_masking:
        cargs.append("--nomask")
    if update_masking:
        cargs.append("--update_mask")
    if no_bet:
        cargs.append("--nobet")
    if bg_threshold is not None:
        cargs.extend([
            "--bgthreshold",
            str(bg_threshold)
        ])
    if dimest_technique is not None:
        cargs.extend([
            "--dimest",
            dimest_technique
        ])
    if separate_variance_normalization:
        cargs.append("--sep_vn")
    if disable_migp:
        cargs.append("--disableMigp")
    if num_internal_eigenmaps is not None:
        cargs.extend([
            "--migpN",
            str(num_internal_eigenmaps)
        ])
    if migp_shuffle:
        cargs.append("--migp_shuffle")
    if migp_factor is not None:
        cargs.extend([
            "--migp_factor",
            str(migp_factor)
        ])
    if num_ics is not None:
        cargs.extend([
            "-n",
            str(num_ics)
        ])
    if nonlinearity is not None:
        cargs.extend([
            "--nl",
            nonlinearity
        ])
    if covar_weights is not None:
        cargs.extend([
            "--covarweight",
            execution.input_file(covar_weights)
        ])
    if eps_error is not None:
        cargs.extend([
            "--eps",
            str(eps_error)
        ])
    if eps_rank1_error is not None:
        cargs.extend([
            "--epsS",
            str(eps_rank1_error)
        ])
    if max_iters is not None:
        cargs.extend([
            "--maxit",
            str(max_iters)
        ])
    if max_restarts is not None:
        cargs.extend([
            "--maxrestart",
            str(max_restarts)
        ])
    if mm_threshold is not None:
        cargs.extend([
            "--mmthresh",
            str(mm_threshold)
        ])
    if no_mixture_modeling:
        cargs.append("--no_mm")
    if ic_components_file is not None:
        cargs.extend([
            "--ICs",
            execution.input_file(ic_components_file)
        ])
    if mixing_matrix_file is not None:
        cargs.extend([
            "--mix",
            execution.input_file(mixing_matrix_file)
        ])
    if session_modes_file is not None:
        cargs.extend([
            "--smode",
            execution.input_file(session_modes_file)
        ])
    if component_filter is not None:
        cargs.extend([
            "-f",
            component_filter
        ])
    if background_image is not None:
        cargs.extend([
            "--bgimage",
            execution.input_file(background_image)
        ])
    if tr_seconds is not None:
        cargs.extend([
            "--tr",
            str(tr_seconds)
        ])
    if log_power_calc:
        cargs.append("--logPower")
    if time_domain_design_matrix is not None:
        cargs.extend([
            "--Tdes",
            execution.input_file(time_domain_design_matrix)
        ])
    if time_domain_t_contrast_matrix is not None:
        cargs.extend([
            "--Tcon",
            execution.input_file(time_domain_t_contrast_matrix)
        ])
    if subject_domain_design_matrix is not None:
        cargs.extend([
            "--Sdes",
            execution.input_file(subject_domain_design_matrix)
        ])
    if subject_domain_t_contrast_matrix is not None:
        cargs.extend([
            "--Scon",
            execution.input_file(subject_domain_t_contrast_matrix)
        ])
    if output_unmixing_matrix:
        cargs.append("--Ounmix")
    if output_stats:
        cargs.append("--Ostats")
    if output_pca:
        cargs.append("--Opca")
    if output_whitening:
        cargs.append("--Owhite")
    if output_original_ics:
        cargs.append("--Oorig")
    if output_mean_volume:
        cargs.append("--Omean")
    if version:
        cargs.append("-V")
    if copyright_:
        cargs.append("--copyright")
    if help_:
        cargs.append("-h")
    if debug:
        cargs.append("--debug")
    if report_maps is not None:
        cargs.extend([
            "--report_maps",
            report_maps
        ])
    if keep_meanvol:
        cargs.append("--keep_meanvol")
    ret = MelodicOutputs(
        root=execution.output_file("."),
        report_file=execution.output_file(output_directory + "/report.html") if (output_directory is not None) else None,
        ics_output_file=execution.output_file(output_directory + "/melodic_IC.nii.gz") if (output_directory is not None) else None,
        mix_output_file=execution.output_file(output_directory + "/melodic_mix") if (output_directory is not None) else None,
        temporal_mode_file=execution.output_file(output_directory + "/melodic_Tmodes") if (output_directory is not None) else None,
        melodic_report_directory=execution.output_file(output_directory + "/melodic_report") if (output_directory is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MELODIC_METADATA",
    "MelodicOutputs",
    "melodic",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FEATREGAPPLY_METADATA = Metadata(
    id="30c4e6513395f39bf385f62de8b909c189f6d029.boutiques",
    name="featregapply",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class FeatregapplyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `featregapply(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_directory: OutputPathType
    """Directory where the output files will be stored"""


def featregapply(
    feat_directory: str,
    force_flag: bool = False,
    cleanup_flag: bool = False,
    upsample_trilinear: InputPathType | None = None,
    upsample_spline: InputPathType | None = None,
    standard_space_res: float | None = None,
    exclude_filtered_func_flag: bool = False,
    runner: Runner | None = None,
) -> FeatregapplyOutputs:
    """
    Apply registration from FEAT analysis to other images.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        feat_directory: FEAT directory from which registration will be taken.
        force_flag: Force featregapply to run even if it has already been run\
            on this FEAT directory.
        cleanup_flag: Cleanup, i.e. remove all featregapply output.
        upsample_trilinear: Upsample functional-space image to standard space\
            using trilinear interpolation.
        upsample_spline: Upsample functional-space image to standard space\
            using spline (like sinc) interpolation.
        standard_space_res: Specify the standard space resolution for melodic\
            (e.g. 3 for 3mm).
        exclude_filtered_func_flag: Exclude filtered func when processing\
            melodic directories (for FEAT directories filtered func is never\
            processed).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FeatregapplyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FEATREGAPPLY_METADATA)
    cargs = []
    cargs.append("featregapply")
    cargs.append(feat_directory)
    if force_flag:
        cargs.append("-f")
    if cleanup_flag:
        cargs.append("-c")
    if upsample_trilinear is not None:
        cargs.extend([
            "-l",
            execution.input_file(upsample_trilinear)
        ])
    if upsample_spline is not None:
        cargs.extend([
            "-s",
            execution.input_file(upsample_spline)
        ])
    if standard_space_res is not None:
        cargs.extend([
            "-r",
            str(standard_space_res)
        ])
    if exclude_filtered_func_flag:
        cargs.append("-e")
    ret = FeatregapplyOutputs(
        root=execution.output_file("."),
        output_directory=execution.output_file(feat_directory + "/reg_standard"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FEATREGAPPLY_METADATA",
    "FeatregapplyOutputs",
    "featregapply",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_SYNTHESIZE_METADATA = Metadata(
    id="814a030fd63ed19fbe9b03dd6aaaa8fcb42ba91b.boutiques",
    name="3dSynthesize",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dSynthesizeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_synthesize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v_3d_synthesize(
    c_bucket: InputPathType,
    matrix: InputPathType,
    select_: str,
    prefix: str,
    dry_flag: bool = False,
    tr: float | None = None,
    cenfill: typing.Literal["zero", "nbhr", "none"] | None = None,
    runner: Runner | None = None,
) -> V3dSynthesizeOutputs:
    """
    Reads a '-cbucket' dataset and a '.xmat.1D' matrix from 3dDeconvolve, and
    synthesizes a fit dataset using selected sub-bricks and matrix columns.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        c_bucket: Input dataset from 3dDeconvolve via the '-cbucket' option.
        matrix: Matrix file from 3dDeconvolve via the '-x1D' option.
        select_: Select columns from the matrix and corresponding sub-bricks\
            from the cbucket. Can use forms like 'baseline', 'polort', 'allfunc',\
            'allstim', 'all', 'something', or numbers/ranges.
        prefix: Output result into dataset with the specified name.
        dry_flag: Don't compute the output, just check the inputs.
        tr: Set TR in the output to the specified value.
        cenfill: How censored time points from 3dDeconvolve will be filled\
            (options: 'zero', 'nbhr', 'none').
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dSynthesizeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_SYNTHESIZE_METADATA)
    cargs = []
    cargs.append("3dSynthesize")
    cargs.extend([
        "-cbucket",
        execution.input_file(c_bucket)
    ])
    cargs.extend([
        "-matrix",
        execution.input_file(matrix)
    ])
    cargs.extend([
        "-select",
        select_
    ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    if dry_flag:
        cargs.append("-dry")
    if tr is not None:
        cargs.extend([
            "-TR",
            str(tr)
        ])
    if cenfill is not None:
        cargs.extend([
            "-cenfill",
            cenfill
        ])
    ret = V3dSynthesizeOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dSynthesizeOutputs",
    "V_3D_SYNTHESIZE_METADATA",
    "v_3d_synthesize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSLREGISTER_METADATA = Metadata(
    id="81e352f9cdac0df3ddc26c66eafd2de97cd00277.boutiques",
    name="fslregister",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class FslregisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fslregister(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_reg_file: OutputPathType
    """Output registration file."""
    output_resliced_volume: OutputPathType | None
    """Resliced output volume."""
    output_fsl_matrix: OutputPathType | None
    """FSL format output registration matrix."""
    lta_output: OutputPathType | None
    """Registration output in LTA format."""
    output_template: OutputPathType | None
    """Output template."""


def fslregister(
    subjid: str,
    mov_vol: str,
    reg_file: str,
    fsl_matrix: str | None = None,
    init_fsl_matrix: str | None = None,
    no_init_xfm: bool = False,
    niters: float | None = None,
    dof: float | None = None,
    bins: float | None = None,
    cost: str | None = None,
    max_angle: float | None = None,
    no_new_schedule: bool = False,
    no_allow_swap: bool = False,
    no_trans: bool = False,
    bet_mov: bool = False,
    bet_fvalue: float | None = None,
    bet_func: bool = False,
    bet_ref: bool = False,
    frame: float | None = None,
    mid_frame: bool = False,
    freesurfer_volume: str | None = None,
    template_output: str | None = None,
    output_volume: str | None = None,
    verbose: float | None = None,
    tmp_dir: str | None = None,
    no_cleanup: bool = False,
    no_log: bool = False,
    version: bool = False,
    help_: bool = False,
    lta_format: str | None = None,
    runner: Runner | None = None,
) -> FslregisterOutputs:
    """
    Registers a volume to its FreeSurfer anatomical using FSL's FLIRT and creates a
    FreeSurfer register.dat file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjid: Id of the subject as found in SUBJECTS_DIR.
        mov_vol: Volume identifier of the movable volume.
        reg_file: Output registration file.
        fsl_matrix: Output registration matrix in FSL format.
        init_fsl_matrix: Supply initial FSL matrix file.
        no_init_xfm: Do not initialize based on header geometry.
        niters: Number of iterations for FLIRT.
        dof: Degrees of freedom in the FLIRT registration.
        bins: Number of bins for FLIRT.
        cost: Cost function for FLIRT.
        max_angle: Maximum search angle for FLIRT.
        no_new_schedule: Disable new schedule in FLIRT.
        no_allow_swap: Do not allow swap dimension of positive determinant\
            input volumes.
        no_trans: Do not perform translation-only registration prior to full.
        bet_mov: Perform brain extraction on movable volume.
        bet_fvalue: f value for BET.
        bet_func: Run betfunc instead of bet.
        bet_ref: Perform brain extraction on reference volume.
        frame: Frame number for registration.
        mid_frame: Use the middle frame of the volume.
        freesurfer_volume: Use FreeSurfer volid as reference volume.
        template_output: Save template.
        output_volume: Have FLIRT reslice movable volume to target.
        verbose: FLIRT verbosity level.
        tmp_dir: Temporary directory to use.
        no_cleanup: Do not delete temporary files.
        no_log: Do not produce a log file.
        version: Print version and exit.
        help_: Print help and exit.
        lta_format: Output registration in LTA format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslregisterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSLREGISTER_METADATA)
    cargs = []
    cargs.append("fslregister")
    cargs.extend([
        "--s",
        subjid
    ])
    cargs.extend([
        "--mov",
        mov_vol
    ])
    cargs.extend([
        "--reg",
        reg_file
    ])
    if fsl_matrix is not None:
        cargs.extend([
            "--fslmat",
            fsl_matrix
        ])
    if init_fsl_matrix is not None:
        cargs.extend([
            "--initfslmat",
            init_fsl_matrix
        ])
    if no_init_xfm:
        cargs.append("--noinitxfm")
    if niters is not None:
        cargs.extend([
            "--niters",
            str(niters)
        ])
    if dof is not None:
        cargs.extend([
            "--dof",
            str(dof)
        ])
    if bins is not None:
        cargs.extend([
            "--bins",
            str(bins)
        ])
    if cost is not None:
        cargs.extend([
            "--cost",
            cost
        ])
    if max_angle is not None:
        cargs.extend([
            "--maxangle",
            str(max_angle)
        ])
    if no_new_schedule:
        cargs.append("--no-new-schedule")
    if no_allow_swap:
        cargs.append("--no-allow-swap")
    if no_trans:
        cargs.append("--no-trans")
    if bet_mov:
        cargs.append("--betmov")
    if bet_fvalue is not None:
        cargs.extend([
            "--betfvalue",
            str(bet_fvalue)
        ])
    if bet_func:
        cargs.append("--betfunc")
    if bet_ref:
        cargs.append("--betref")
    if frame is not None:
        cargs.extend([
            "--frame",
            str(frame)
        ])
    if mid_frame:
        cargs.append("--mid-frame")
    if freesurfer_volume is not None:
        cargs.extend([
            "--fsvol",
            freesurfer_volume
        ])
    if template_output is not None:
        cargs.extend([
            "--template-out",
            template_output
        ])
    if output_volume is not None:
        cargs.extend([
            "--out",
            output_volume
        ])
    if verbose is not None:
        cargs.extend([
            "--verbose",
            str(verbose)
        ])
    if tmp_dir is not None:
        cargs.extend([
            "--tmp",
            tmp_dir
        ])
    if no_cleanup:
        cargs.append("--nocleanup")
    if no_log:
        cargs.append("--nolog")
    if version:
        cargs.append("--version")
    if help_:
        cargs.append("--help")
    if lta_format is not None:
        cargs.extend([
            "--lta",
            lta_format
        ])
    ret = FslregisterOutputs(
        root=execution.output_file("."),
        output_reg_file=execution.output_file(reg_file),
        output_resliced_volume=execution.output_file(output_volume) if (output_volume is not None) else None,
        output_fsl_matrix=execution.output_file(fsl_matrix) if (fsl_matrix is not None) else None,
        lta_output=execution.output_file(lta_format) if (lta_format is not None) else None,
        output_template=execution.output_file(template_output) if (template_output is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSLREGISTER_METADATA",
    "FslregisterOutputs",
    "fslregister",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA = Metadata(
    id="c11dd7c0d9840d40017d52a654120cd6141ec439.boutiques",
    name="ANTSUseLandmarkImagesToGetAffineTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class AntsuseLandmarkImagesToGetAffineTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `antsuse_landmark_images_to_get_affine_transform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    affine_transform_matrix: OutputPathType
    """The output is the affine transformation matrix file."""


def antsuse_landmark_images_to_get_affine_transform(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    This tool computes an affine transform (rigid or affine) from labeled landmark
    images. It requires input images to be N-ary, in the same physical space as the
    images you want to register, and to have the same landmark points defined within
    them. Landmarks are defined from the center of mass of the labels in the input
    images. ITK-snap can be used to generate the label images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        fixed_image: The fixed image containing labeled landmarks (N-ary\
            image).
        moving_image: The moving image containing labeled landmarks (N-ary\
            image).
        transform_type: Type of transform to compute: 'rigid' or 'affine'.
        output_affine: The output file for the affine transform matrix (e.g.,\
            OutAffine.txt).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetAffineTransformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA)
    cargs = []
    cargs.append("ANTSUseLandmarkImagesToGetAffineTransform")
    cargs.append(execution.input_file(fixed_image))
    cargs.append(execution.input_file(moving_image))
    cargs.append(transform_type)
    cargs.append(output_affine)
    ret = AntsuseLandmarkImagesToGetAffineTransformOutputs(
        root=execution.output_file("."),
        affine_transform_matrix=execution.output_file(output_affine),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA",
    "AntsuseLandmarkImagesToGetAffineTransformOutputs",
    "antsuse_landmark_images_to_get_affine_transform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_AFNITO_ANALYZE_METADATA = Metadata(
    id="8e009ca0525a816d98d8d648c7fbae2324df58e2.boutiques",
    name="3dAFNItoANALYZE",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dAfnitoAnalyzeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_afnito_analyze(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_hdr_file: OutputPathType
    """ANALYZE header file for each sub-brick"""
    output_img_file: OutputPathType
    """ANALYZE image file for each sub-brick"""
    output_4d_hdr_file: OutputPathType
    """Single ANALYZE header file if using -4D option"""
    output_4d_img_file: OutputPathType
    """Single ANALYZE image file if using -4D option"""


def v_3d_afnito_analyze(
    output_name: str,
    afni_dataset: InputPathType,
    v_4d_option: bool = False,
    orient_option: str | None = None,
    runner: Runner | None = None,
) -> V3dAfnitoAnalyzeOutputs:
    """
    Writes AFNI dataset to ANALYZE 7.5 format .hdr/.img file pairs.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        output_name: Output ANALYZE file base name (e.g., aname).
        afni_dataset: Input AFNI dataset.
        v_4d_option: Write all data to one big ANALYZE file pair named\
            aname.hdr/aname.img.
        orient_option: Flip the dataset to a different orientation when writing\
            to ANALYZE files (e.g., LPI).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dAfnitoAnalyzeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_AFNITO_ANALYZE_METADATA)
    cargs = []
    cargs.append("3dAFNItoANALYZE")
    if v_4d_option:
        cargs.append("-4D")
    if orient_option is not None:
        cargs.extend([
            "-orient",
            orient_option
        ])
    cargs.append(output_name)
    cargs.append(execution.input_file(afni_dataset))
    ret = V3dAfnitoAnalyzeOutputs(
        root=execution.output_file("."),
        output_hdr_file=execution.output_file(output_name + "_[INDEX].hdr"),
        output_img_file=execution.output_file(output_name + "_[INDEX].img"),
        output_4d_hdr_file=execution.output_file(output_name + ".hdr"),
        output_4d_img_file=execution.output_file(output_name + ".img"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dAfnitoAnalyzeOutputs",
    "V_3D_AFNITO_ANALYZE_METADATA",
    "v_3d_afnito_analyze",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMSTAT_METADATA = Metadata(
    id="c59c01eb2a12e7d9a2066403a73878b709e55aa5.boutiques",
    name="imstat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ImstatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imstat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mean_output: OutputPathType | None
    """Mean of pixel-wise statistics for the collection of 2D images"""
    sdev_output: OutputPathType | None
    """Standard deviation of pixel-wise statistics for the collection of 2D
    images"""


def imstat(
    image_files: list[InputPathType],
    no_label: bool = False,
    quiet: bool = False,
    pixstat_prefix: str | None = None,
    runner: Runner | None = None,
) -> ImstatOutputs:
    """
    Calculation of statistics of one or more images.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        image_files: Input image file(s).
        no_label: Don't write labels on each file's summary line.
        quiet: Don't print statistics for each file.
        pixstat_prefix: If more than one image file is given, then\
            'prefix.mean' and 'prefix.sdev' will be written as the pixel-wise\
            statistics images of the whole collection. These images will be in the\
            'flim' floating point format. [This option only works on 2D images!].
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImstatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMSTAT_METADATA)
    cargs = []
    cargs.append("imstat")
    if no_label:
        cargs.append("-nolabel")
    if quiet:
        cargs.append("-quiet")
    if pixstat_prefix is not None:
        cargs.extend([
            "-pixstat",
            pixstat_prefix
        ])
    cargs.extend([execution.input_file(f) for f in image_files])
    ret = ImstatOutputs(
        root=execution.output_file("."),
        mean_output=execution.output_file(pixstat_prefix + ".mean") if (pixstat_prefix is not None) else None,
        sdev_output=execution.output_file(pixstat_prefix + ".sdev") if (pixstat_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMSTAT_METADATA",
    "ImstatOutputs",
    "imstat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_EM_REGISTER_METADATA = Metadata(
    id="9edc027d219877b8b87c7bf356e57518fe060c1a.boutiques",
    name="mri_em_register",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriEmRegisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_em_register(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform_file: OutputPathType
    """Transform in LTA format"""


def mri_em_register(
    input_volume: InputPathType,
    template_gca: InputPathType,
    output_transform: str,
    distance: float | None = None,
    nomap: bool = False,
    flash: bool = False,
    mask: InputPathType | None = None,
    skull: bool = False,
    uns: float | None = None,
    diag: str | None = None,
    debug_voxel: list[float] | None = None,
    debug_label: float | None = None,
    tr: float | None = None,
    te: float | None = None,
    alpha: float | None = None,
    example: list[InputPathType] | None = None,
    samples: str | None = None,
    fsamples: str | None = None,
    nsamples: str | None = None,
    contrast: bool = False,
    flash_parms: InputPathType | None = None,
    transonly: bool = False,
    write_mean: str | None = None,
    prior: float | None = None,
    spacing: float | None = None,
    scales: float | None = None,
    novar: bool = False,
    dt: float | None = None,
    tol: float | None = None,
    center: bool = False,
    noscale: bool = False,
    noiscale: bool = False,
    num_transforms: float | None = None,
    area: float | None = None,
    nlarea: float | None = None,
    levels: float | None = None,
    intensity: float | None = None,
    reduce: float | None = None,
    n_samples: float | None = None,
    norm: str | None = None,
    trans: float | None = None,
    steps: float | None = None,
    long_reg: str | None = None,
    cpfile: InputPathType | None = None,
    translation_vector: list[float] | None = None,
    rotation_vector: list[float] | None = None,
    xform: str | None = None,
    blur: float | None = None,
    diagno: bool = False,
    s: float | None = None,
    max_angle: float | None = None,
    niters: float | None = None,
    write_iters: float | None = None,
    ctl_point_pct: float | None = None,
    momentum: float | None = None,
    threads: float | None = None,
    runner: Runner | None = None,
) -> MriEmRegisterOutputs:
    """
    This program creates a transform in lta format.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: Input brain volume.
        template_gca: Template GCA file.
        output_transform: Output transform name.
        distance: Distance.
        nomap: No map.
        flash: Use FLASH forward model to predict intensity values.
        mask: Use volname as a mask.
        skull: Align to atlas containing skull (uns=5).
        uns: Align to atlas containing skull setting unknown_nbr_spacing =\
            nbrspacing.
        diag: Open diagfile for writing.
        debug_voxel: Debug voxel (x, y, z).
        debug_label: Debug label (label).
        tr: Use TR msec.
        te: Use TE msec.
        alpha: Use alpha degrees.
        example: Use T1 and seg as example T1 and segmentations respectively.
        samples: Write control points to fname.
        fsamples: Write transformed control points to fname.
        nsamples: Write transformed normalization control points to fname.
        contrast: Use contrast to find labels.
        flash_parms: Use FLASH forward model and tissue parms in parameterfile\
            to predict intensity values.
        transonly: Only compute translation parameters.
        write_mean: Write GCA means to fname.
        prior: Use prior threshold min_prior.
        spacing: Use max GCA spacing.
        scales: Find optimal linear transform over int scales.
        novar: Do not use variance estimates.
        dt: DT parameter.
        tol: Tolerance.
        center: Use GCA centroid as origin of transform.
        noscale: Disable scaling.
        noiscale: Disable intensity scaling.
        num_transforms: Find a total of num_xforms linear transforms.
        area: Area.
        nlarea: Non-linear area.
        levels: Levels.
        intensity: Intensity.
        reduce: Reduce input images nreductions times before aligning.
        n_samples: Using n samples of GCA.
        norm: Normalize intensity and write to fname.
        trans: Setting max translation search range to be max_trans.
        steps: Taking max_angles angular steps.
        long_reg: Longitudinal: read previously computed atlas xform and apply\
            registration long_reg.
        cpfile: Read manually defined control points from cpfile.
        translation_vector: Translation vector (tx, ty, tz).
        rotation_vector: Rotation vector (rx, ry, rz).
        xform: Using previously computed transform xform.
        blur: Blurring input image with sigma=blur_sigma.
        diagno: Diago flag (unspecified function).
        s: Max angles.
        max_angle: Max angle for rotational search in radians (def=15 deg).
        niters: Niterations = niters.
        write_iters: Write iterations = write_iters.
        ctl_point_pct: Use top pct percent wm points as control points.
        momentum: Set momentum.
        threads: Number of threads (nompthreads).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriEmRegisterOutputs`).
    """
    if example is not None and (len(example) != 2): 
        raise ValueError(f"Length of 'example' must be 2 but was {len(example)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_EM_REGISTER_METADATA)
    cargs = []
    cargs.append("mri_em_register")
    cargs.append(execution.input_file(input_volume))
    cargs.append(execution.input_file(template_gca))
    cargs.append(output_transform)
    if distance is not None:
        cargs.extend([
            "-dist",
            str(distance)
        ])
    if nomap:
        cargs.append("-nomap")
    if flash:
        cargs.append("-flash")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if skull:
        cargs.append("-skull")
    if uns is not None:
        cargs.extend([
            "-uns",
            str(uns)
        ])
    if diag is not None:
        cargs.extend([
            "-diag",
            diag
        ])
    if debug_voxel is not None:
        cargs.extend([
            "-debug_voxel",
            *map(str, debug_voxel)
        ])
    if debug_label is not None:
        cargs.extend([
            "-debug_label",
            str(debug_label)
        ])
    if tr is not None:
        cargs.extend([
            "-tr",
            str(tr)
        ])
    if te is not None:
        cargs.extend([
            "-te",
            str(te)
        ])
    if alpha is not None:
        cargs.extend([
            "-alpha",
            str(alpha)
        ])
    if example is not None:
        cargs.extend([
            "-example",
            *[execution.input_file(f) for f in example]
        ])
    if samples is not None:
        cargs.extend([
            "-samples",
            samples
        ])
    if fsamples is not None:
        cargs.extend([
            "-fsamples",
            fsamples
        ])
    if nsamples is not None:
        cargs.extend([
            "-nsamples",
            nsamples
        ])
    if contrast:
        cargs.append("-contrast")
    if flash_parms is not None:
        cargs.extend([
            "-flash_parms",
            execution.input_file(flash_parms)
        ])
    if transonly:
        cargs.append("-transonly")
    if write_mean is not None:
        cargs.extend([
            "-write_mean",
            write_mean
        ])
    if prior is not None:
        cargs.extend([
            "-prior",
            str(prior)
        ])
    if spacing is not None:
        cargs.extend([
            "-spacing",
            str(spacing)
        ])
    if scales is not None:
        cargs.extend([
            "-scales",
            str(scales)
        ])
    if novar:
        cargs.append("-novar")
    if dt is not None:
        cargs.extend([
            "-dt",
            str(dt)
        ])
    if tol is not None:
        cargs.extend([
            "-tol",
            str(tol)
        ])
    if center:
        cargs.append("-center")
    if noscale:
        cargs.append("-noscale")
    if noiscale:
        cargs.append("-noiscale")
    if num_transforms is not None:
        cargs.extend([
            "-num",
            str(num_transforms)
        ])
    if area is not None:
        cargs.extend([
            "-area",
            str(area)
        ])
    if nlarea is not None:
        cargs.extend([
            "-nlarea",
            str(nlarea)
        ])
    if levels is not None:
        cargs.extend([
            "-levels",
            str(levels)
        ])
    if intensity is not None:
        cargs.extend([
            "-intensity",
            str(intensity)
        ])
    if reduce is not None:
        cargs.extend([
            "-reduce",
            str(reduce)
        ])
    if n_samples is not None:
        cargs.extend([
            "-nsamples",
            str(n_samples)
        ])
    if norm is not None:
        cargs.extend([
            "-norm",
            norm
        ])
    if trans is not None:
        cargs.extend([
            "-trans",
            str(trans)
        ])
    if steps is not None:
        cargs.extend([
            "-steps",
            str(steps)
        ])
    if long_reg is not None:
        cargs.extend([
            "-l",
            long_reg
        ])
    if cpfile is not None:
        cargs.extend([
            "-f",
            execution.input_file(cpfile)
        ])
    if translation_vector is not None:
        cargs.extend([
            "-d",
            *map(str, translation_vector)
        ])
    if rotation_vector is not None:
        cargs.extend([
            "-r",
            *map(str, rotation_vector)
        ])
    if xform is not None:
        cargs.extend([
            "-t",
            xform
        ])
    if blur is not None:
        cargs.extend([
            "-b",
            str(blur)
        ])
    if diagno:
        cargs.append("-v")
    if s is not None:
        cargs.extend([
            "-s",
            str(s)
        ])
    if max_angle is not None:
        cargs.extend([
            "-max_angle",
            str(max_angle)
        ])
    if niters is not None:
        cargs.extend([
            "-n",
            str(niters)
        ])
    if write_iters is not None:
        cargs.extend([
            "-w",
            str(write_iters)
        ])
    if ctl_point_pct is not None:
        cargs.extend([
            "-p",
            str(ctl_point_pct)
        ])
    if momentum is not None:
        cargs.extend([
            "-m",
            str(momentum)
        ])
    if threads is not None:
        cargs.extend([
            "-threads",
            str(threads)
        ])
    ret = MriEmRegisterOutputs(
        root=execution.output_file("."),
        output_transform_file=execution.output_file(output_transform + ".lta"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_EM_REGISTER_METADATA",
    "MriEmRegisterOutputs",
    "mri_em_register",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_LOCAL_BISTAT_METADATA = Metadata(
    id="e90dc29ecfbe470e814874e0a612f6a2f98a8350.boutiques",
    name="3dLocalBistat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dLocalBistatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_local_bistat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_head: OutputPathType
    """Output dataset header for AFNI format"""
    output_brik: OutputPathType
    """Output dataset BRIK for AFNI format"""


def v_3d_local_bistat(
    nbhd: str,
    stats: list[str],
    prefix: str,
    dataset1: InputPathType,
    dataset2: InputPathType,
    mask: InputPathType | None = None,
    automask: bool = False,
    weight: InputPathType | None = None,
    histpow: float | None = None,
    histbin: float | None = None,
    hclip1: list[str] | None = None,
    hclip2: list[str] | None = None,
    runner: Runner | None = None,
) -> V3dLocalBistatOutputs:
    """
    Compute statistics between 2 datasets at each voxel based on a local
    neighborhood.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        nbhd: Specifies the neighborhood around each voxel for statistics\
            calculation. Types include: SPHERE(r), RECT(a,b,c), RHDD(r), TOHD(r).
        stats: Statistic to compute in the region around each voxel. Multiple\
            options allowed. Includes: pearson, spearman, quadrant, mutinfo,\
            normuti, jointent, hellinger, crU, crM, crA, L2slope, L1slope, num,\
            ALL.
        prefix: Prefix of the output dataset.
        dataset1: The first input dataset (e.g. data1.nii).
        dataset2: The second input dataset (e.g. data2.nii).
        mask: Read in a dataset to use as a mask. Non-zero voxels define the\
            mask region.
        automask: Compute the mask as in program 3dAutomask. Mutually exclusive\
            with -mask.
        weight: Use dataset as a weight (applies to pearson).
        histpow: Sets the exponent for the number of bins in the histogram used\
            for Hellinger, Mutual Information, and Correlation Ratio statistics.
        histbin: Sets the number of bins directly in the histogram used for\
            Hellinger, Mutual Information, and Correlation Ratio statistics.
        hclip1: Clip dataset1 to lie between specified values.
        hclip2: Clip dataset2 to lie between specified values.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dLocalBistatOutputs`).
    """
    if hclip1 is not None and (len(hclip1) != 2): 
        raise ValueError(f"Length of 'hclip1' must be 2 but was {len(hclip1)}")
    if hclip2 is not None and (len(hclip2) != 2): 
        raise ValueError(f"Length of 'hclip2' must be 2 but was {len(hclip2)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_LOCAL_BISTAT_METADATA)
    cargs = []
    cargs.append("3dLocalBistat")
    cargs.extend([
        "-nbhd",
        nbhd
    ])
    cargs.extend([
        "-stat",
        *stats
    ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if automask:
        cargs.append("-automask")
    if weight is not None:
        cargs.extend([
            "-weight",
            execution.input_file(weight)
        ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    if histpow is not None:
        cargs.extend([
            "-histpow",
            str(histpow)
        ])
    if histbin is not None:
        cargs.extend([
            "-histbin",
            str(histbin)
        ])
    if hclip1 is not None:
        cargs.extend([
            "-hclip1",
            *hclip1
        ])
    if hclip2 is not None:
        cargs.extend([
            "-hclip2",
            *hclip2
        ])
    cargs.append(execution.input_file(dataset1))
    cargs.append(execution.input_file(dataset2))
    ret = V3dLocalBistatOutputs(
        root=execution.output_file("."),
        output_head=execution.output_file(prefix + "+orig.HEAD"),
        output_brik=execution.output_file(prefix + "+orig.BRIK"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dLocalBistatOutputs",
    "V_3D_LOCAL_BISTAT_METADATA",
    "v_3d_local_bistat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CORRECT_SEGMENTATIONS_METADATA = Metadata(
    id="bc3ea4572fe4ebef8155dbeace0ea841cbc2a21c.boutiques",
    name="mri_correct_segmentations",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCorrectSegmentationsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_correct_segmentations(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_correct_segmentations(
    input_file_1: InputPathType,
    input_file_2: InputPathType,
    runner: Runner | None = None,
) -> MriCorrectSegmentationsOutputs:
    """
    Tool for correcting automated infant segmentations.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_file_1: First input file for correction (e.g. segmentation file).
        input_file_2: Second input file for correction (e.g. reference file).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCorrectSegmentationsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CORRECT_SEGMENTATIONS_METADATA)
    cargs = []
    cargs.append("mri_correct_segmentations")
    cargs.append(execution.input_file(input_file_1))
    cargs.append(execution.input_file(input_file_2))
    ret = MriCorrectSegmentationsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CORRECT_SEGMENTATIONS_METADATA",
    "MriCorrectSegmentationsOutputs",
    "mri_correct_segmentations",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

METADATA_REMOVE_PROVENANCE_METADATA = Metadata(
    id="477bbc80e00e51f900e75625d9c13d64da765e46.boutiques",
    name="metadata-remove-provenance",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class MetadataRemoveProvenanceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `metadata_remove_provenance(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metadata_remove_provenance(
    input_file: str,
    output_file: str,
    runner: Runner | None = None,
) -> MetadataRemoveProvenanceOutputs:
    """
    Remove provenance information from file metadata.
    
    Removes the provenance metadata fields added by workbench during processing.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        input_file: the file to remove provenance information from.
        output_file: output - the name to save the modified file as.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetadataRemoveProvenanceOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METADATA_REMOVE_PROVENANCE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metadata-remove-provenance")
    cargs.append(input_file)
    cargs.append(output_file)
    ret = MetadataRemoveProvenanceOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "METADATA_REMOVE_PROVENANCE_METADATA",
    "MetadataRemoveProvenanceOutputs",
    "metadata_remove_provenance",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

T4IMG_4DFP_METADATA = Metadata(
    id="57020555f03db47bd822a3164b665430a2020fd9.boutiques",
    name="t4img_4dfp",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class T4img4dfpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `t4img_4dfp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    transformed_image: OutputPathType | None
    """The transformed output image file in 4dfp format."""


def t4img_4dfp(
    t4file: InputPathType,
    imgfile: InputPathType,
    outfile: str | None = None,
    runner: Runner | None = None,
) -> T4img4dfpOutputs:
    """
    Transforms a 4dfp image using a specified t4 file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        t4file: Transformation matrix file (t4 file format).
        imgfile: Input image file (4dfp format).
        outfile: Output image file (optional, defaults to <imgfile>t if not\
            provided).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `T4img4dfpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(T4IMG_4DFP_METADATA)
    cargs = []
    cargs.append("t4img_4dfp")
    cargs.append(execution.input_file(t4file))
    cargs.append(execution.input_file(imgfile))
    if outfile is not None:
        cargs.append(outfile)
    ret = T4img4dfpOutputs(
        root=execution.output_file("."),
        transformed_image=execution.output_file(outfile + ".4dfp.img") if (outfile is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "T4IMG_4DFP_METADATA",
    "T4img4dfpOutputs",
    "t4img_4dfp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_EDIT_SEGMENTATION_WITH_SURFACES_METADATA = Metadata(
    id="b7676f431b8610a951f255b96b0e4e26a937a332.boutiques",
    name="mri_edit_segmentation_with_surfaces",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriEditSegmentationWithSurfacesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_edit_segmentation_with_surfaces(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume_file: OutputPathType
    """Edited output volume"""


def mri_edit_segmentation_with_surfaces(
    aseg_name: InputPathType,
    surface_dir: str,
    norm_volume: InputPathType,
    output_volume: str,
    label_file: InputPathType | None = None,
    hypo_flag: typing.Literal["1", "0"] | None = None,
    cerebellum_flag: typing.Literal["1", "0"] | None = None,
    cortex_flag: typing.Literal["1", "0"] | None = None,
    annotation_file: InputPathType | None = None,
    runner: Runner | None = None,
) -> MriEditSegmentationWithSurfacesOutputs:
    """
    This program edits an aseg with the surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        aseg_name: The aseg file to be edited.
        surface_dir: Directory containing surface files.
        norm_volume: Normalized volume file.
        output_volume: Output volume file.
        label_file: Limit calculations to specified label.
        hypo_flag: Turn hypointensity editing on/off (1=on, 0=off).
        cerebellum_flag: Turn cerebellum editing on/off (1=on, 0=off).
        cortex_flag: Turn cortex editing on/off (1=on, 0=off).
        annotation_file: Compute properties for each label in the annotation\
            file separately.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriEditSegmentationWithSurfacesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_EDIT_SEGMENTATION_WITH_SURFACES_METADATA)
    cargs = []
    cargs.append("mri_edit_segmentation_with_surfaces")
    cargs.append(execution.input_file(aseg_name))
    cargs.append(surface_dir)
    cargs.append(execution.input_file(norm_volume))
    cargs.append(output_volume)
    if label_file is not None:
        cargs.extend([
            "-l",
            execution.input_file(label_file)
        ])
    if hypo_flag is not None:
        cargs.extend([
            "-hypo",
            hypo_flag
        ])
    if cerebellum_flag is not None:
        cargs.extend([
            "-cerebellum",
            cerebellum_flag
        ])
    if cortex_flag is not None:
        cargs.extend([
            "-cortex",
            cortex_flag
        ])
    if annotation_file is not None:
        cargs.extend([
            "-a",
            execution.input_file(annotation_file)
        ])
    ret = MriEditSegmentationWithSurfacesOutputs(
        root=execution.output_file("."),
        output_volume_file=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_EDIT_SEGMENTATION_WITH_SURFACES_METADATA",
    "MriEditSegmentationWithSurfacesOutputs",
    "mri_edit_segmentation_with_surfaces",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

P2DSETSTAT_METADATA = Metadata(
    id="425596bb580582f82bf97f7e288cb9c38b721949.boutiques",
    name="p2dsetstat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class P2dsetstatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `p2dsetstat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stat_value: OutputPathType
    """The converted statistic value."""


def p2dsetstat(
    dataset: str,
    pvalue: float,
    onesided: bool = False,
    quiet: bool = False,
    runner: Runner | None = None,
) -> P2dsetstatOutputs:
    """
    Convert a p-value to a statistic of choice with reference to a specific dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dataset: Specify a dataset DDD and, if it has multiple sub-bricks, the\
            [i]th subbrick with the statistic of interest MUST be selected\
            explicitly; note the use of quotation marks around the brick selector\
            (because of the square-brackets). 'i' can be either a number or a\
            string label selector.
        pvalue: Input p-value P, which MUST be in the interval [0,1].
        onesided: One-sided test.
        quiet: Output only the final statistic value.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `P2dsetstatOutputs`).
    """
    if not (0 <= pvalue <= 1): 
        raise ValueError(f"'pvalue' must be between 0 <= x <= 1 but was {pvalue}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(P2DSETSTAT_METADATA)
    cargs = []
    cargs.append("p2dsetstat")
    cargs.extend([
        "-inset",
        dataset
    ])
    cargs.extend([
        "-pval",
        str(pvalue)
    ])
    if onesided:
        cargs.append("-1sided")
    if quiet:
        cargs.append("-quiet")
    ret = P2dsetstatOutputs(
        root=execution.output_file("."),
        stat_value=execution.output_file("stdout"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "P2DSETSTAT_METADATA",
    "P2dsetstatOutputs",
    "p2dsetstat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_LOCAL_ACF_METADATA = Metadata(
    id="f02de9f266ae864cf54889181b4c744098070407.boutiques",
    name="3dLocalACF",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dLocalAcfOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_local_acf(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output dataset with ACF estimates"""


def v_3d_local_acf(
    prefix: str,
    input_file: InputPathType,
    neighborhood: str | None = None,
    mask_file: InputPathType | None = None,
    auto_mask: bool = False,
    runner: Runner | None = None,
) -> V3dLocalAcfOutputs:
    """
    Estimate the spatial AutoCorrelation Function (ACF) locally in a neighborhood
    around each voxel.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        prefix: Prefix for output dataset.
        input_file: Input time series dataset.
        neighborhood: Neighborhood specification (e.g., SPHERE(25)).
        mask_file: Dataset to mask the analysis.
        auto_mask: Automatically generate brain mask from input dataset.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dLocalAcfOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_LOCAL_ACF_METADATA)
    cargs = []
    cargs.append("3dLocalACF")
    cargs.extend([
        "-prefix",
        prefix
    ])
    cargs.append(execution.input_file(input_file))
    if neighborhood is not None:
        cargs.extend([
            "-nbhd",
            neighborhood
        ])
    if mask_file is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_file)
        ])
    if auto_mask:
        cargs.append("-automask")
    ret = V3dLocalAcfOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix + ".nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dLocalAcfOutputs",
    "V_3D_LOCAL_ACF_METADATA",
    "v_3d_local_acf",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TCK2CONNECTOME_METADATA = Metadata(
    id="d7266b72589a1c26b9abb6847aa3a2ba49db356b.boutiques",
    name="tck2connectome",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class Tck2connectomeConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class Tck2connectomeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tck2connectome(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    connectome_out: OutputPathType
    """the output .csv file containing edge weights"""
    out_assignments: OutputPathType | None
    """output the node assignments of each streamline to a file; this can be
    used subsequently e.g. by the command connectome2tck """


def tck2connectome(
    tracks_in: InputPathType,
    nodes_in: InputPathType,
    connectome_out: str,
    assignment_end_voxels: bool = False,
    assignment_radial_search: float | None = None,
    assignment_reverse_search: float | None = None,
    assignment_forward_search: float | None = None,
    assignment_all_voxels: bool = False,
    scale_length: bool = False,
    scale_invlength: bool = False,
    scale_invnodevol: bool = False,
    scale_file: InputPathType | None = None,
    symmetric: bool = False,
    zero_diagonal: bool = False,
    stat_edge: str | None = None,
    tck_weights_in: InputPathType | None = None,
    keep_unassigned: bool = False,
    out_assignments: str | None = None,
    vector: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tck2connectomeConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Tck2connectomeOutputs:
    """
    Generate a connectome matrix from a streamlines file and a node parcellation
    image.
    
    
    
    References:
    
    If using the default streamline-parcel assignment mechanism (or
    -assignment_radial_search option): Smith, R. E.; Tournier, J.-D.; Calamante,
    F. & Connelly, A. The effects of SIFT on the reproducibility and biological
    accuracy of the structural connectome. NeuroImage, 2015, 104, 253-265
    
    If using -scale_invlength or -scale_invnodevol options: Hagmann, P.;
    Cammoun, L.; Gigandet, X.; Meuli, R.; Honey, C.; Wedeen, V. & Sporns, O.
    Mapping the Structural Core of Human Cerebral Cortex. PLoS Biology 6(7),
    e159.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks_in: the input track file.
        nodes_in: the input node parcellation image.
        connectome_out: the output .csv file containing edge weights.
        assignment_end_voxels: use a simple voxel lookup value at each\
            streamline endpoint.
        assignment_radial_search: perform a radial search from each streamline\
            endpoint to locate the nearest node. Argument is the maximum radius in\
            mm; if no node is found within this radius, the streamline endpoint is\
            not assigned to any node. Default search distance is 4mm.
        assignment_reverse_search: traverse from each streamline endpoint\
            inwards along the streamline, in search of the last node traversed by\
            the streamline. Argument is the maximum traversal length in mm (set to\
            0 to allow search to continue to the streamline midpoint).
        assignment_forward_search: project the streamline forwards from the\
            endpoint in search of a parcellation node voxel. Argument is the\
            maximum traversal length in mm.
        assignment_all_voxels: assign the streamline to all nodes it intersects\
            along its length (note that this means a streamline may be assigned to\
            more than two nodes, or indeed none at all).
        scale_length: scale each contribution to the connectome edge by the\
            length of the streamline.
        scale_invlength: scale each contribution to the connectome edge by the\
            inverse of the streamline length.
        scale_invnodevol: scale each contribution to the connectome edge by the\
            inverse of the two node volumes.
        scale_file: scale each contribution to the connectome edge according to\
            the values in a vector file.
        symmetric: Make matrices symmetric on output.
        zero_diagonal: Set matrix diagonal to zero on output.
        stat_edge: statistic for combining the values from all streamlines in\
            an edge into a single scale value for that edge (options are:\
            sum,mean,min,max; default=sum).
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        keep_unassigned: By default, the program discards the information\
            regarding those streamlines that are not successfully assigned to a\
            node pair. Set this option to keep these values (will be the first\
            row/column in the output matrix).
        out_assignments: output the node assignments of each streamline to a\
            file; this can be used subsequently e.g. by the command connectome2tck.
        vector: output a vector representing connectivities from a given seed\
            point to target nodes, rather than a matrix of node-node connectivities.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tck2connectomeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCK2CONNECTOME_METADATA)
    cargs = []
    cargs.append("tck2connectome")
    if assignment_end_voxels:
        cargs.append("-assignment_end_voxels")
    if assignment_radial_search is not None:
        cargs.extend([
            "-assignment_radial_search",
            str(assignment_radial_search)
        ])
    if assignment_reverse_search is not None:
        cargs.extend([
            "-assignment_reverse_search",
            str(assignment_reverse_search)
        ])
    if assignment_forward_search is not None:
        cargs.extend([
            "-assignment_forward_search",
            str(assignment_forward_search)
        ])
    if assignment_all_voxels:
        cargs.append("-assignment_all_voxels")
    if scale_length:
        cargs.append("-scale_length")
    if scale_invlength:
        cargs.append("-scale_invlength")
    if scale_invnodevol:
        cargs.append("-scale_invnodevol")
    if scale_file is not None:
        cargs.extend([
            "-scale_file",
            execution.input_file(scale_file)
        ])
    if symmetric:
        cargs.append("-symmetric")
    if zero_diagonal:
        cargs.append("-zero_diagonal")
    if stat_edge is not None:
        cargs.extend([
            "-stat_edge",
            stat_edge
        ])
    if tck_weights_in is not None:
        cargs.extend([
            "-tck_weights_in",
            execution.input_file(tck_weights_in)
        ])
    if keep_unassigned:
        cargs.append("-keep_unassigned")
    if out_assignments is not None:
        cargs.extend([
            "-out_assignments",
            out_assignments
        ])
    if vector:
        cargs.append("-vector")
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(tracks_in))
    cargs.append(execution.input_file(nodes_in))
    cargs.append(connectome_out)
    ret = Tck2connectomeOutputs(
        root=execution.output_file("."),
        connectome_out=execution.output_file(connectome_out),
        out_assignments=execution.output_file(out_assignments) if (out_assignments is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TCK2CONNECTOME_METADATA",
    "Tck2connectomeConfig",
    "Tck2connectomeOutputs",
    "tck2connectome",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_SPH2SURF_METADATA = Metadata(
    id="b6f756a7470f5c010ca807e14f237f111ed456b6.boutiques",
    name="mri-sph2surf",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriSph2surfOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_sph2surf(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output surface data file in the form outstem-lh.w (or rh)."""


def mri_sph2surf(
    instem: str,
    outstem: str,
    hemi: str,
    subject: str,
    offset: float | None = 0,
    svitdir: str | None = "/usr/local/freesurfer/subjects/subject/svit",
    umask: str | None = None,
    verbose: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MriSph2surfOutputs:
    """
    Converts spherical functional data to surface data in the FreeSurfer processing
    pipeline.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        instem: Input stem of a bfloat file. The full input file name must take\
            the form instem-lh_000.bfloat (or rh).
        outstem: Output stem for the resulting file. The output file will have\
            the name outstem-lh.w (or rh).
        hemi: Specifies the hemisphere for processing. Acceptable values are\
            'lh' or 'rh'.
        subject: Specifies the subject identifier for the FreeSurfer processing\
            pipeline.
        offset: Zero-based plane/frame number. Default is 0.
        svitdir: Directory for svit. Default is\
            '/usr/local/freesurfer/subjects/subject/svit'.
        umask: Specifies a new user mask.
        verbose: Enable verbose output.
        version: Show version information.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriSph2surfOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_SPH2SURF_METADATA)
    cargs = []
    cargs.append("mri-sph2surf")
    cargs.extend([
        "-i",
        instem
    ])
    cargs.extend([
        "-o",
        outstem
    ])
    cargs.extend([
        "-hemi",
        hemi
    ])
    cargs.extend([
        "-s",
        subject
    ])
    if offset is not None:
        cargs.extend([
            "-offset",
            str(offset)
        ])
    if svitdir is not None:
        cargs.extend([
            "-svitdir",
            svitdir
        ])
    if umask is not None:
        cargs.extend([
            "-umask",
            umask
        ])
    if verbose:
        cargs.append("-verbose")
    if version:
        cargs.append("-version")
    ret = MriSph2surfOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(outstem + "-" + hemi + ".w"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_SPH2SURF_METADATA",
    "MriSph2surfOutputs",
    "mri_sph2surf",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CA_NORMALIZE_METADATA = Metadata(
    id="822a8e4a63033035b2c8f0da0858f4e52e7344bd.boutiques",
    name="mri_ca_normalize",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCaNormalizeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_ca_normalize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    normalized_output: OutputPathType
    """Normalized output volume in mgh format."""


def mri_ca_normalize(
    input_brain_volumes: list[InputPathType],
    atlas_file: InputPathType,
    xform_file: InputPathType,
    output_volumes: list[str],
    seg_file: InputPathType | None = None,
    sigma_value: float | None = None,
    fsamples_file: InputPathType | None = None,
    dilate_iters: float | None = None,
    nsamples_file: InputPathType | None = None,
    mask_vol: InputPathType | None = None,
    control_points_file: InputPathType | None = None,
    fonly_file: InputPathType | None = None,
    diag_file: InputPathType | None = None,
    debug_voxel_coords: list[float] | None = None,
    debug_node_coords: list[float] | None = None,
    tr_value: float | None = None,
    te_value: float | None = None,
    alpha_value: float | None = None,
    example_mri_vol: InputPathType | None = None,
    extra_norm_pctl: float | None = None,
    prior_threshold: float | None = None,
    n_regions: float | None = None,
    verbose_value: float | None = None,
    top_percent: float | None = None,
    novar_flag: bool = False,
    renorm_file: InputPathType | None = None,
    flash_flag: bool = False,
    runner: Runner | None = None,
) -> MriCaNormalizeOutputs:
    """
    This program creates a normalized volume using the brain volume and an input gca
    file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_brain_volumes: Input brain volume(s). Can specify multiple\
            inputs.
        atlas_file: Atlas file in GCA format.
        xform_file: Transform file in LTA format.
        output_volumes: Output volume(s) in either mgh or mgz format. Can\
            specify multiple outputs.
        seg_file: Aseg file to help normalization.
        sigma_value: Smoothing sigma for bias field if control points specified\
            (default=4).
        fsamples_file: Write control points to filename.
        dilate_iters: Dilate the brain mask niters times before masking.
        nsamples_file: Write transformed normalization control points to\
            filename.
        mask_vol: Use mri_vol to mask input.
        control_points_file: Define control points from filename.
        fonly_file: Only use control points from filename.
        diag_file: Write to log file.
        debug_voxel_coords: Debug voxel. Needs x, y, z coordinates.
        debug_node_coords: Debug node. Needs x, y, z coordinates.
        tr_value: Set TR in msec.
        te_value: Set TE in msec.
        alpha_value: Set alpha in radians.
        example_mri_vol: Use T1 (mri_vol) and segmentation as example.
        extra_norm_pctl: Use 1+pct and 1-pct to widen the range of T1 values.
        prior_threshold: Use prior threshold t (default=.6).
        n_regions: Use n regions/struct for normalization.
        verbose_value: Used for debugging and diagnostics.
        top_percent: Use top p percent (default=.25) white matter points as\
            control points.
        novar_flag: Do not use variance estimates.
        renorm_file: Renormalize using predicted intensity values in mri_vol.
        flash_flag: Use FLASH forward model to predict intensity values.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCaNormalizeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CA_NORMALIZE_METADATA)
    cargs = []
    cargs.append("mri_ca_normalize")
    cargs.extend([execution.input_file(f) for f in input_brain_volumes])
    cargs.append(execution.input_file(atlas_file))
    cargs.append(execution.input_file(xform_file))
    cargs.extend(output_volumes)
    if seg_file is not None:
        cargs.extend([
            "-seg",
            execution.input_file(seg_file)
        ])
    if sigma_value is not None:
        cargs.extend([
            "-sigma",
            str(sigma_value)
        ])
    if fsamples_file is not None:
        cargs.extend([
            "-fsamples",
            execution.input_file(fsamples_file)
        ])
    if dilate_iters is not None:
        cargs.extend([
            "-dilate",
            str(dilate_iters)
        ])
    if nsamples_file is not None:
        cargs.extend([
            "-nsamples",
            execution.input_file(nsamples_file)
        ])
    if mask_vol is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_vol)
        ])
    if control_points_file is not None:
        cargs.extend([
            "-f",
            execution.input_file(control_points_file)
        ])
    if fonly_file is not None:
        cargs.extend([
            "-fonly",
            execution.input_file(fonly_file)
        ])
    if diag_file is not None:
        cargs.extend([
            "-diag",
            execution.input_file(diag_file)
        ])
    if debug_voxel_coords is not None:
        cargs.extend([
            "-debug_voxel",
            *map(str, debug_voxel_coords)
        ])
    if debug_node_coords is not None:
        cargs.extend([
            "-debug_node",
            *map(str, debug_node_coords)
        ])
    if tr_value is not None:
        cargs.extend([
            "-tr",
            str(tr_value)
        ])
    if te_value is not None:
        cargs.extend([
            "-te",
            str(te_value)
        ])
    if alpha_value is not None:
        cargs.extend([
            "-alpha",
            str(alpha_value)
        ])
    if example_mri_vol is not None:
        cargs.extend([
            "-example",
            execution.input_file(example_mri_vol)
        ])
    if extra_norm_pctl is not None:
        cargs.extend([
            "-extra_norm",
            str(extra_norm_pctl)
        ])
    if prior_threshold is not None:
        cargs.extend([
            "-prior",
            str(prior_threshold)
        ])
    if n_regions is not None:
        cargs.extend([
            "-n",
            str(n_regions)
        ])
    if verbose_value is not None:
        cargs.extend([
            "-v",
            str(verbose_value)
        ])
    if top_percent is not None:
        cargs.extend([
            "-p",
            str(top_percent)
        ])
    if novar_flag:
        cargs.append("-novar")
    if renorm_file is not None:
        cargs.extend([
            "-renorm",
            execution.input_file(renorm_file)
        ])
    if flash_flag:
        cargs.append("-flash")
    ret = MriCaNormalizeOutputs(
        root=execution.output_file("."),
        normalized_output=execution.output_file("output.mgh"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CA_NORMALIZE_METADATA",
    "MriCaNormalizeOutputs",
    "mri_ca_normalize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

XHEMIREG_METADATA = Metadata(
    id="f01b4255c8c371fe9d8cf7dac45f58046ba01896.boutiques",
    name="xhemireg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class XhemiregOutputs(typing.NamedTuple):
    """
    Output object returned when calling `xhemireg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def xhemireg(
    subject: str,
    output_dir: str | None = None,
    map_lh: bool = False,
    map_rh: bool = False,
    perform_reg: bool = False,
    tal_compute: bool = False,
    no_tal_compute: bool = False,
    tal_estimate: bool = False,
    no_tal_estimate: bool = False,
    gcaprep: str | None = None,
    threads: float | None = None,
    version: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> XhemiregOutputs:
    """
    Tool for hemisphere registration in neuroimaging.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: Subject ID for the hemisphere registration process.
        output_dir: Output directory for the hemisphere registration.
        map_lh: Map from left hemisphere to right hemisphere.
        map_rh: Map from right hemisphere to left hemisphere.
        perform_reg: Perform registration to create sphere.reg.
        tal_compute: Recompute Talairach registration.
        no_tal_compute: Do not perform Talairach registration.
        tal_estimate: Compute estimate of Talairach registration from unflipped\
            registration.
        no_tal_estimate: Do not perform estimation of Talairach registration.
        gcaprep: Prepare GCA for training symmetrical GCA atlases.
        threads: Number of threads used, applicable with --gcaprep option.
        version: Print version and exit.
        help_: Print help and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `XhemiregOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(XHEMIREG_METADATA)
    cargs = []
    cargs.append("xhemireg")
    cargs.extend([
        "--s",
        subject
    ])
    if output_dir is not None:
        cargs.extend([
            "--o",
            output_dir
        ])
    if map_lh:
        cargs.append("--lh")
    if map_rh:
        cargs.append("--rh")
    if perform_reg:
        cargs.append("--reg")
    if tal_compute:
        cargs.append("--tal-compute")
    if no_tal_compute:
        cargs.append("--no-tal-compute")
    if tal_estimate:
        cargs.append("--tal-estimate")
    if no_tal_estimate:
        cargs.append("--no-tal-estimate")
    if gcaprep is not None:
        cargs.extend([
            "--gcaprep",
            gcaprep
        ])
    if threads is not None:
        cargs.extend([
            "--threads",
            str(threads)
        ])
    if version:
        cargs.append("--version")
    if help_:
        cargs.append("--help")
    ret = XhemiregOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "XHEMIREG_METADATA",
    "XhemiregOutputs",
    "xhemireg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_NWARP_XYZ_METADATA = Metadata(
    id="5a16d2bee46238f64451cbe6e0fea267dd5ecf53.boutiques",
    name="3dNwarpXYZ",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dNwarpXyzOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_nwarp_xyz(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Warped XYZ coordinates output file"""


def v_3d_nwarp_xyz(
    warp_spec: str,
    xyzfile: InputPathType,
    runner: Runner | None = None,
) -> V3dNwarpXyzOutputs:
    """
    Transforms the DICOM xyz coordinates in the input XYZfile.1D based on specified
    warp.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        warp_spec: Warp specification as in 3dNwarpApply.
        xyzfile: XYZ coordinate file containing 3 columns.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dNwarpXyzOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_NWARP_XYZ_METADATA)
    cargs = []
    cargs.append("3dNwarpXYZ")
    cargs.append("[OPTIONS]")
    cargs.append("-nwarp")
    cargs.extend([
        "-nwarp",
        warp_spec
    ])
    cargs.append(execution.input_file(xyzfile))
    cargs.append(">")
    cargs.append("[OUTPUT_FILE]")
    ret = V3dNwarpXyzOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file("[OUTPUT_FILE]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dNwarpXyzOutputs",
    "V_3D_NWARP_XYZ_METADATA",
    "v_3d_nwarp_xyz",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_MOTION_CORRECT_FSL_METADATA = Metadata(
    id="e4429f150e08917f7317e164c448c83b94352eed.boutiques",
    name="mri_motion_correct.fsl",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriMotionCorrectFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_motion_correct_fsl(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    corrected_output: OutputPathType
    """Motion corrected output MRI image"""


def mri_motion_correct_fsl(
    input_image: InputPathType,
    output_image: str,
    runner: Runner | None = None,
) -> MriMotionCorrectFslOutputs:
    """
    Tool for motion correction of MRI images using FSL.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_image: Input MRI image to be motion corrected.
        output_image: Output corrected MRI image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriMotionCorrectFslOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_MOTION_CORRECT_FSL_METADATA)
    cargs = []
    cargs.append("mri_motion_correct.fsl")
    cargs.append(execution.input_file(input_image))
    cargs.append(output_image)
    cargs.append("[OPTIONS]")
    ret = MriMotionCorrectFslOutputs(
        root=execution.output_file("."),
        corrected_output=execution.output_file(output_image + ".nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_MOTION_CORRECT_FSL_METADATA",
    "MriMotionCorrectFslOutputs",
    "mri_motion_correct_fsl",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA = Metadata(
    id="c7361c4e340d4044ba58e2e93a667ceb9c6989fd.boutiques",
    name="volume-warpfield-affine-regression",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class VolumeWarpfieldAffineRegressionFlirtOut:
    """
    write output as a flirt matrix rather than a world coordinate transform.
    """
    source_volume: str
    """the volume you want to apply the transform to"""
    target_volume: str
    """the target space you want the transformed volume to match"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-flirt-out")
        cargs.append(self.source_volume)
        cargs.append(self.target_volume)
        return cargs


class VolumeWarpfieldAffineRegressionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_warpfield_affine_regression(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_warpfield_affine_regression(
    warpfield: str,
    affine_out: str,
    opt_roi_roi_vol: InputPathType | None = None,
    opt_fnirt_source_volume: str | None = None,
    flirt_out: VolumeWarpfieldAffineRegressionFlirtOut | None = None,
    runner: Runner | None = None,
) -> VolumeWarpfieldAffineRegressionOutputs:
    """
    Regress affine from warpfield.
    
    For all voxels in the warpfield, do a regression that predicts the post-warp
    coordinate from the source coordinate. When -roi is specified, only consider
    voxels with a value greater than 0 in <roi-vol>.
    
    The default is to expect the warpfield to be in relative world coordinates
    (mm space), and to write the output as a world affine (mm space to mm
    space). If you are using FSL-created files and utilities, specify -fnirt and
    -flirt as needed, as their coordinate conventions are different.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        warpfield: the input warpfield.
        affine_out: output - the output affine file.
        opt_roi_roi_vol: only consider voxels within a mask (e.g., a brain\
            mask): the mask volume.
        opt_fnirt_source_volume: input is a fnirt warpfield: the source volume\
            used when generating the fnirt warpfield.
        flirt_out: write output as a flirt matrix rather than a world\
            coordinate transform.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWarpfieldAffineRegressionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-warpfield-affine-regression")
    cargs.append(warpfield)
    cargs.append(affine_out)
    if opt_roi_roi_vol is not None:
        cargs.extend([
            "-roi",
            execution.input_file(opt_roi_roi_vol)
        ])
    if opt_fnirt_source_volume is not None:
        cargs.extend([
            "-fnirt",
            opt_fnirt_source_volume
        ])
    if flirt_out is not None:
        cargs.extend(flirt_out.run(execution))
    ret = VolumeWarpfieldAffineRegressionOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA",
    "VolumeWarpfieldAffineRegressionFlirtOut",
    "VolumeWarpfieldAffineRegressionOutputs",
    "volume_warpfield_affine_regression",
]
